#include "urlget.h"
#include "utility.h"
#include "batch_dialog.h"

static void batch_dialog_decide_sensitive(BatchDialog* bd)
{
	gboolean responsive = FALSE;
	UrlPart  up;

	const gchar* str = gtk_entry_get_text (GTK_ENTRY(bd->entry));

	url_part (&up, str);

	if (up.protocol_beg && up.address_beg && up.name_beg) {
		for (; *str ; ++str) {
			if ( *str == '*' ) {
				responsive=TRUE;
				break;
			}
		}
	}
	gtk_dialog_set_response_sensitive (GTK_DIALOG (bd->self),
	                                   GTK_RESPONSE_OK,
	                                   responsive);
}

// signal handler --------------------------------------------------------------
static gboolean entry_key_release(GtkWidget* w, GdkEventKey* ek, BatchDialog* bd)
{
	batch_dialog_decide_sensitive (bd);
	return FALSE;
}

static void entry_paste_clipboard(GtkEntry *entry, BatchDialog* bd)
{
	batch_dialog_decide_sensitive (bd);
}

static void radio1_clicked(GtkWidget* widget, BatchDialog* bd)
{
	gtk_widget_set_sensitive (bd->spin_from, TRUE);
	gtk_widget_set_sensitive (bd->spin_to, TRUE);
	gtk_widget_set_sensitive (bd->spin_width, TRUE);
	gtk_widget_set_sensitive (bd->entry_a, FALSE);
	gtk_widget_set_sensitive (bd->entry_z, FALSE);
}

static void radio2_clicked(GtkWidget* widget, BatchDialog* bd)
{
	gtk_widget_set_sensitive (bd->spin_from, FALSE);
	gtk_widget_set_sensitive (bd->spin_to, FALSE);
	gtk_widget_set_sensitive (bd->spin_width, FALSE);
	gtk_widget_set_sensitive (bd->entry_a, TRUE);
	gtk_widget_set_sensitive (bd->entry_z, TRUE);
}

void batch_dialog_init (BatchDialog* bd, GtkWindow* parent)
{
	GtkWidget* vbox;
	GtkWidget* hbox;
	GtkWidget* label;
	GtkAdjustment* spinner_adj;
	GtkSizeGroup*  size_group;
	GtkSizeGroup*  size_group_label;

	// initial dialog
	bd->self = gtk_dialog_new_with_buttons (_("Batch mission"),
	                                        parent,
	                                        GTK_DIALOG_MODAL,
	                                        GTK_STOCK_CANCEL,
	                                        GTK_RESPONSE_CANCEL,
	                                        GTK_STOCK_OK,
	                                        GTK_RESPONSE_OK,
	                                        NULL);
	gtk_window_set_resizable (GTK_WINDOW (bd->self),
	                          FALSE);
	gtk_dialog_set_response_sensitive (GTK_DIALOG(bd->self),
	                                   GTK_RESPONSE_OK,
	                                   FALSE);
	gtk_container_set_border_width (GTK_CONTAINER(bd->self), 5);

	// pack widgets
	vbox = GTK_DIALOG (bd->self)->vbox;
	gtk_box_set_spacing (GTK_BOX(vbox), 5);

	size_group = gtk_size_group_new (GTK_SIZE_GROUP_HORIZONTAL);
	size_group_label = gtk_size_group_new (GTK_SIZE_GROUP_HORIZONTAL);

	// URL entry
	hbox = gtk_hbox_new (FALSE, 5);
	label = gtk_label_new_with_mnemonic (_("_URL"));
	bd->entry = gtk_entry_new ();
	g_signal_connect (bd->entry, "key-release-event",
	                  G_CALLBACK(entry_key_release), bd );
	g_signal_connect_after (bd->entry, "paste-clipboard",
	                        G_CALLBACK(entry_paste_clipboard), bd );
	gtk_box_pack_start (GTK_BOX(hbox), label, FALSE,FALSE,0);
	gtk_box_pack_start (GTK_BOX(hbox), bd->entry, TRUE, TRUE,0);
	gtk_box_pack_start (GTK_BOX(vbox), hbox, FALSE,FALSE,0);

	hbox = gtk_hbox_new (FALSE, 5);
	label = gtk_label_new (_("Example:"));
	gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 0);
	label = gtk_label_new ("http://example.dot.com/path_name/pre*.jpg");
	gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 0);
	gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, 0);

	// spin "From"
	hbox = gtk_hbox_new (FALSE, 5);
	bd->radio_button = gtk_radio_button_new_with_mnemonic (NULL, _("_From :"));
	g_signal_connect (bd->radio_button, "clicked",
	                  G_CALLBACK(radio1_clicked), bd);
	gtk_box_pack_start (GTK_BOX(hbox),
	                    bd->radio_button,
	                    FALSE, FALSE, 0);
	spinner_adj = (GtkAdjustment *) gtk_adjustment_new (0.0, 0.0,
	                                                    99999.0, 1.0,
	                                                    5.0, 5.0);
	bd->spin_from = gtk_spin_button_new (spinner_adj, 1.0, 0);
	gtk_box_pack_start (GTK_BOX(hbox), bd->spin_from,
	                    FALSE, FALSE, 0);
	gtk_size_group_add_widget (size_group, bd->radio_button);
	gtk_size_group_add_widget (size_group, bd->spin_from);

	// spin "To"
	label = gtk_label_new_with_mnemonic (_(" _To :"));
	gtk_box_pack_start (GTK_BOX(hbox), label,
	                    FALSE, FALSE, 0);
	spinner_adj = (GtkAdjustment *) gtk_adjustment_new (0.0, 0.0,
	                                                    99999.0, 1.0,
	                                                    5.0, 5.0);
	bd->spin_to = gtk_spin_button_new (spinner_adj, 1.0, 0);
	gtk_label_set_mnemonic_widget (GTK_LABEL(label), bd->spin_to);
	gtk_box_pack_start (GTK_BOX(hbox),
	                    bd->spin_to,
	                    FALSE, FALSE, 0);
	gtk_size_group_add_widget (size_group_label, label);
	gtk_size_group_add_widget (size_group, bd->spin_to);

	// spin "Width"
	label = gtk_label_new_with_mnemonic (_("_Wildcard bytes :"));
	gtk_box_pack_start (GTK_BOX(hbox),
	                    label,
	                    FALSE, FALSE, 0);
	spinner_adj = (GtkAdjustment *) gtk_adjustment_new (2.0, 1.0,
	                                                    20.0, 1.0,
	                                                    5.0, 5.0);
	bd->spin_width = gtk_spin_button_new (spinner_adj, 1.0, 0);
	gtk_label_set_mnemonic_widget (GTK_LABEL(label), bd->spin_width);
	gtk_box_pack_start(GTK_BOX(hbox), bd->spin_width, FALSE, FALSE, 0);
	gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);

	hbox = gtk_hbox_new (FALSE, 5);
	bd->radio_button = gtk_radio_button_new_with_mnemonic_from_widget (GTK_RADIO_BUTTON(bd->radio_button),
	                                                                   _("F_rom :"));
	g_signal_connect (bd->radio_button, "clicked",
	                  G_CALLBACK(radio2_clicked), bd);
	gtk_box_pack_start (GTK_BOX(hbox),
	                    bd->radio_button,
	                    FALSE, FALSE, 0);
	bd->entry_a = gtk_entry_new ();
	gtk_entry_set_text (GTK_ENTRY(bd->entry_a), "a");
	gtk_entry_set_max_length (GTK_ENTRY(bd->entry_a), 1);
	gtk_entry_set_width_chars (GTK_ENTRY(bd->entry_a), 2);
	gtk_widget_set_sensitive (bd->entry_a, FALSE);
	gtk_box_pack_start (GTK_BOX(hbox),
	                    bd->entry_a,
	                    FALSE, FALSE, 0);
	gtk_size_group_add_widget (size_group, bd->radio_button);
	gtk_size_group_add_widget (size_group, bd->entry_a);

	label = gtk_label_new_with_mnemonic ("   ");
	gtk_box_pack_start (GTK_BOX (hbox),
	                    label,
	                    FALSE, FALSE, 0);
	bd->entry_z = gtk_entry_new ();
	gtk_entry_set_text (GTK_ENTRY(bd->entry_z), "z");
	gtk_entry_set_max_length (GTK_ENTRY(bd->entry_z), 1);
	gtk_entry_set_width_chars (GTK_ENTRY(bd->entry_z), 2);
	gtk_widget_set_sensitive (bd->entry_z, FALSE);
	gtk_box_pack_start (GTK_BOX(hbox),
	                    bd->entry_z,
	                    FALSE, FALSE, 0);
	gtk_size_group_add_widget (size_group_label, label);
	gtk_size_group_add_widget (size_group, bd->entry_z);
	gtk_box_pack_start (GTK_BOX(vbox), hbox, FALSE, FALSE, 0);

	label = gtk_label_new (_("Warning : case sensitive"));
	gtk_box_pack_start (GTK_BOX(hbox), label, FALSE, FALSE, 0);
	gtk_widget_show_all (vbox);
}

void batch_dialog_clear (BatchDialog* bd)
{
	gtk_widget_destroy (bd->self);
}

gint batch_dialog_run (BatchDialog* bd)
{
	gint response;

	response = gtk_dialog_run (GTK_DIALOG (bd->self));

	if ( response==GTK_RESPONSE_CANCEL || response==GTK_RESPONSE_DELETE_EVENT)
		return BATCH_MODE_NONE;

	if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (bd->radio_button)) )
		return BATCH_MODE_CHAR;

	return BATCH_MODE_NUM;
}

const gchar* batch_dialog_get_pattern (BatchDialog* bd)
{
	return gtk_entry_get_text (GTK_ENTRY(bd->entry));
}

gchar batch_dialog_get_char_min (BatchDialog* bd)
{
	return *gtk_entry_get_text (GTK_ENTRY(bd->entry_a));
}

gchar batch_dialog_get_char_max (BatchDialog* bd)
{
	return *gtk_entry_get_text (GTK_ENTRY(bd->entry_z));
}

gint batch_dialog_get_int_min (BatchDialog* bd)
{
	return gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON(bd->spin_from));
}

gint batch_dialog_get_int_max (BatchDialog* bd)
{
	return gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON(bd->spin_to));
}

gint batch_dialog_get_int_width (BatchDialog* bd)
{
	return gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON(bd->spin_width));
}

