/*
	category.h



	2003   Plus Huang
 */

#ifndef __CATEGORY_H
#define __CATEGORY_H

#include <glib.h>
#include "category_setting.h"
#include "download_list_view.h"

#ifdef __cplusplus
extern "C" {
#endif

//
//   ProxySetting
//   |
//   `--DownloadSetting
//      |
//      `--CategorySetting
//         |
//         `-- Category
//

#define CATEGORY(o)          ((Category*)(o))
#define CATEGORY_TAG         "category"

#define CATEGORY_WAITING_STORE_TAG    "waiting_store"
#define CATEGORY_COMPLETED_STORE_TAG  "completed_store"
#define CATEGORY_RECYCLED_STORE_TAG   "recycled_store"

typedef struct _Category      Category;

struct _Category {
	CategorySetting     setting;

	// iter in CategoryStore.list_store
	// iter in CategoryStore.tree_store
	gpointer            category_store;
	GtkTreeIter         iter_list_store;
	GtkTreeIter         iter_tree_store;
	gboolean            valid_iter;

	GStaticRecMutex     mutex;
	gpointer            thread_list;
	gint                n_thread;
	gint                reference_count;

	DownloadListView    waiting_view;
	DownloadListView    completed_view;
	DownloadListView    recycled_view;
	DownloadStore       waiting_store;
	DownloadStore       completed_store;
	DownloadStore       recycled_store;
};

enum CATEGORY_CHILDREN {
	CATEGORY_WAITING,
	CATEGORY_COMPLETED,
	CATEGORY_RECYCLED,
	CATEGORY_ALL
};

Category* category_new ();
Category* category_new_from_setting (CategorySetting* cd);

void category_ref (Category* ca);
void category_unref (Category* ca);

void category_apply_setting (Category* ca);
// call this when CATEGORY_COL_STATE or other display data changed
void category_changed_with_children (Category* ca, gint flag_child);
#define category_changed(ca)    category_changed_with_children(ca, CATEGORY_ALL)

#define category_lock(cate)        g_static_rec_mutex_lock (&(cate)->mutex)
#define category_unlock(cate)      g_static_rec_mutex_unlock (&(cate)->mutex)

// thread manager for Category
gint     category_thread_count_request (Category* ca);
gboolean category_thread_activate (Category* ca);
void     category_thread_destroy (Category* ca);
void     category_thread_stop (Category* category);
void     category_thread_remove (Category* category,
                                 gpointer thread);
void     category_thread_add (Category* category,
                              gpointer thread);

#define category_thread_is_active(cate)   ((cate)->n_thread)

// xml
void category_write_xml (Category* category,
                         FILE* file, int level);

#ifdef __cplusplus
}
#endif

#endif  // End of __CATEGORY_H
