#include <string.h>
#include <stdlib.h>
#include "urlget.h"
#include "category_setting.h"

CategorySetting* category_setting_init (CategorySetting* cd)
{
	download_setting_init(&cd->download);

	cd->download.multiple = 1;  // TRUE
	cd->multiple = 0;           // FALSE
	cd->name = NULL;
	cd->state = CATEGORY_STATE_WAITING;
	cd->n_active_download = 1;
	cd->completed_capacity = 1000;
	cd->recycled_capacity = 1000;

	// column visible
	cd->col_completed = 1;
	cd->col_total     = 1;
	cd->col_speed     = 1;
	cd->col_retry     = 1;
	cd->col_url       = 1;

	// text
	cd->text_filename   = 1;
	cd->text_directory  = 1;
	cd->text_url        = 1;
	cd->text_start_mode = 1;
	cd->text_message    = 1;

	return cd;
}

void category_setting_clear(CategorySetting* cd)
{
	g_free (cd->name);
	download_setting_clear (&cd->download);
}

void category_setting_destroy (CategorySetting* cd)
{
	category_setting_clear (cd);
	g_free (cd);
}

void category_setting_assign (CategorySetting* cd, CategorySetting* src)
{
	download_setting_assign (&cd->download, &src->download);

	if (cd->multiple==0 && src->multiple==0)
		category_setting_set_name (cd, src->name);
	cd->state             = src->state;
	cd->n_active_download = src->n_active_download;
	cd->completed_capacity = src->completed_capacity;
	cd->recycled_capacity  = src->recycled_capacity;

	cd->col_completed = src->col_completed;
	cd->col_total     = src->col_total;
	cd->col_speed     = src->col_speed;
	cd->col_retry     = src->col_retry;
	cd->col_url       = src->col_url;

	cd->text_filename   = src->text_filename;
	cd->text_directory  = src->text_directory;
	cd->text_url        = src->text_url;
	cd->text_start_mode = src->text_start_mode;
	cd->text_message    = src->text_message;
}

void category_setting_parse_xml_attr (CategorySetting* setting,
                                      const char** attr_names,
                                      const char** attr_values)
{
	int ii;

	for (ii=0; attr_names[ii]; ii++) {
		if (strcmp (attr_names[ii], "name")==0)
			setting_set_string (&setting->name, attr_values[ii]);

		if (strcmp (attr_names[ii], "state")==0)
			setting->state = atoi (attr_values[ii]);
		if (strcmp (attr_names[ii], "multiple")==0)
			setting->multiple = atoi (attr_values[ii]);
		if (strcmp (attr_names[ii], "n_active_download")==0)
			setting->n_active_download = atoi (attr_values[ii]);

		if (strcmp (attr_names[ii], "completed_capacity")==0)
			setting->completed_capacity = atoi (attr_values[ii]);
		if (strcmp (attr_names[ii], "recycled_capacity")==0)
			setting->recycled_capacity = atoi (attr_values[ii]);

		if (strcmp (attr_names[ii], "col_completed")==0)
			setting->col_completed = atoi (attr_values[ii]);
		if (strcmp (attr_names[ii], "col_total")==0)
			setting->col_total = atoi (attr_values[ii]);
		if (strcmp (attr_names[ii], "col_speed")==0)
			setting->col_speed = atoi (attr_values[ii]);
		if (strcmp (attr_names[ii], "col_retry")==0)
			setting->col_retry = atoi (attr_values[ii]);
		if (strcmp (attr_names[ii], "col_url")==0)
			setting->col_url = atoi (attr_values[ii]);
		if (strcmp (attr_names[ii], "text_filename")==0)

			setting->text_filename = atoi (attr_values[ii]);
		if (strcmp (attr_names[ii], "text_directory")==0)
			setting->text_directory = atoi (attr_values[ii]);
		if (strcmp (attr_names[ii], "text_url")==0)
			setting->text_url = atoi (attr_values[ii]);
		if (strcmp (attr_names[ii], "text_start_mode")==0)
			setting->text_start_mode = atoi (attr_values[ii]);
		if (strcmp (attr_names[ii], "text_message")==0)
			setting->text_message = atoi (attr_values[ii]);
	}
}

void category_setting_write_xml (CategorySetting* setting,
                                 FILE* file, int level)
{
	guint state = setting->state;

	if (state == CATEGORY_STATE_EXECUTING)
		state = DOWNLOAD_STATE_WAITING;

	fprintf (file,
	         "%*c<" CATEGORY_SETTING_TAG " "         // Start of format string
	         "name=\"%s\" "
	         "state=\"%d\" "
			 "multiple=\"%d\" "
			 "n_active_download=\"%d\" "
	         "completed_capacity=\"%d\" "
	         "recycled_capacity=\"%d\" "
	         "col_completed=\"%d\" "
	         "col_total=\"%d\" "
	         "col_speed=\"%d\" "
	         "col_retry=\"%d\" "
	         "col_url=\"%d\" "
	         "text_filename=\"%d\" "
	         "text_directory=\"%d\" "
	         "text_url=\"%d\" "
	         "text_start_mode=\"%d\" "
	         "text_message=\"%d\">\n",   // End of format string
	         level, ' ',
	         STR_N2E (setting->name),
	         state,
	         setting->multiple,
	         setting->n_active_download,
	         setting->completed_capacity,
	         setting->recycled_capacity,
	         setting->col_completed,
	         setting->col_total,
	         setting->col_speed,
	         setting->col_retry,
	         setting->col_url,
	         setting->text_filename,
	         setting->text_directory,
	         setting->text_url,
	         setting->text_start_mode,
	         setting->text_message
	         );
/*
	fprintf (file, "%*c<column name=\"completed\" visible=\"%d\">\n",
	         level+1, ' ', setting->col_completed);
	fprintf (file, "%*c<column name=\"total\" visible=\"%d\">\n",
	         level+1, ' ', setting->col_total);
	fprintf (file, "%*c<column name=\"speed\" visible=\"%d\">\n",
	         level+1, ' ', setting->col_speed);
*/
	download_setting_write_xml (DOWNLOAD_SETTING (setting), file, level+1);
	fprintf (file, "%*c</" CATEGORY_SETTING_TAG ">\n", level, ' ');
}

