/*
	category_store.c



	2003  Plus Huang
 */

#include <stdlib.h>
#include "category_store.h"
#include "download_thread.h"

void category_store_init (CategoryStore* cs)
{
	g_static_rec_mutex_init (&cs->mutex);
	cs->n_thread = 0;
	cs->n_thread_limit = 1;

	cs->list_store = gtk_list_store_new (CATEGORY_N_COL,
	                                     G_TYPE_POINTER,
	                                     G_TYPE_POINTER,
	                                     G_TYPE_POINTER);

	cs->tree_store = gtk_tree_store_new (CATEGORY_N_COL,
	                                     G_TYPE_POINTER,
	                                     G_TYPE_POINTER,
	                                     G_TYPE_POINTER);
}

void category_store_clear (CategoryStore* cs)
{
	g_object_unref (cs->list_store);
	g_object_unref (cs->tree_store);
}

void category_store_append (CategoryStore* cs,
                            Category* cate)
{
	GtkTreeIter iter_child;

	category_store_lock (cs);

	cate->category_store = cs;
	cate->valid_iter = TRUE;

	// append data to GtkListStore
	gtk_list_store_append (cs->list_store,
	                       &cate->iter_list_store);
	gtk_list_store_set (cs->list_store,
	                    &cate->iter_list_store,
	                    CATEGORY_COL_DATA, cate,
	                    CATEGORY_COL_DOWNLOAD_STORE, &cate->waiting_store,
	                    CATEGORY_COL_DOWNLOAD_VIEW,  &cate->waiting_view,
	                    -1);

	// append data to GtkTreeStore
	gtk_tree_store_append (cs->tree_store,
	                       &cate->iter_tree_store, NULL);
	gtk_tree_store_set (cs->tree_store,
	                    &cate->iter_tree_store,
	                    CATEGORY_COL_DATA, cate,
	                    CATEGORY_COL_DOWNLOAD_STORE, &cate->waiting_store,
	                    CATEGORY_COL_DOWNLOAD_VIEW,  &cate->waiting_view,
	                    -1);
	gtk_tree_store_append (cs->tree_store,
	                       &iter_child,
	                       &cate->iter_tree_store);
	gtk_tree_store_set (cs->tree_store,
	                    &iter_child,
	                    CATEGORY_COL_DATA, cate,
	                    CATEGORY_COL_DOWNLOAD_STORE, &cate->completed_store,
	                    CATEGORY_COL_DOWNLOAD_VIEW,  &cate->completed_view,
	                    -1);
	gtk_tree_store_append (cs->tree_store,
	                       &iter_child,
	                       &cate->iter_tree_store);
	gtk_tree_store_set (cs->tree_store,
                        &iter_child,
	                    CATEGORY_COL_DATA, cate,
	                    CATEGORY_COL_DOWNLOAD_STORE, &cate->recycled_store,
	                    CATEGORY_COL_DOWNLOAD_VIEW,  &cate->recycled_view,
	                    -1);

	category_store_unlock (cs);
}

void category_store_erase (CategoryStore* cs,
                           Category* category)
{
	category_store_lock (cs);

	category->valid_iter = FALSE;
	category_thread_destroy (category);
	category_unref (category);

	gtk_list_store_remove (cs->list_store, &category->iter_list_store);
	gtk_tree_store_remove (cs->tree_store, &category->iter_tree_store);

	category_store_unlock (cs);
}

Category* category_store_get_next (CategoryStore* cs,
                                   Category* category)
{
	GtkTreeIter iter;
	gboolean    valid = FALSE;
	Category*   next_category = NULL;

	if (category == NULL || category->valid_iter==FALSE ) {
		valid = gtk_tree_model_get_iter_first (GTK_TREE_MODEL (cs->list_store),
		                                       &iter);
	}
	else {
		iter = category->iter_list_store;
		valid = gtk_tree_model_iter_next (GTK_TREE_MODEL (cs->list_store),
		                                  &iter);
		if (valid==FALSE) {
			valid = gtk_tree_model_get_iter_first (GTK_TREE_MODEL (cs->list_store),
			                                       &iter);
		}
	}

	if (valid) {
		gtk_tree_model_get (GTK_TREE_MODEL (cs->list_store), &iter,
		                    CATEGORY_COL_DATA, &next_category, -1);
		if (next_category == category)
			return NULL;
	}

	return next_category;
}

// thread operate for CategoryStore
gint category_store_thread_count_request (CategoryStore* cs)
{
	return cs->n_thread_limit - cs->n_thread;
}

void category_store_thread_ref (CategoryStore* cs)
{
	category_store_lock (cs);
	cs->n_thread++;
	printf ("active category %d\n", cs->n_thread);
	category_store_unlock (cs);
}

void category_store_thread_unref (CategoryStore* cs)
{
	category_store_lock (cs);
	cs->n_thread--;
	printf ("active category %d\n", cs->n_thread);
	category_store_unlock (cs);
}

void category_store_thread_activate_1 (CategoryStore* cs,
                                       Category* category)
{
	category_lock (category);
	category_store_lock (cs);

	// try to activate another download thread in category until limit
	if (category_thread_is_active (category) ||
	    category_store_thread_count_request (cs) > 0)
	{
		category_thread_activate (category);
	}

	category_store_unlock (cs);
	category_unlock (category);
}

/*
void category_store_thread_stop_1 (CategoryStore* cs,
                                   Category* category)
{
	Category*    first_category;
	Category*    next_category;

	category_lock (category);
	category_store_lock (cs);

	category_thread_destroy (category);
	cs->n_thread--;

	category_store_unlock (cs);
	category_unlock (category);

	category_store_thread_activate_next (cs, category);
}
*/

void category_store_thread_activate_next (CategoryStore* cs,
                                          Category* category)
{
	Category*    first_category;
	Category*    next_category;

	category_store_lock (cs);

	next_category = first_category = category_store_get_next (cs, category);

	while (next_category) {
		if (category_store_thread_count_request (cs) < 1)
			break;

		category_thread_activate (next_category);

		next_category = category_store_get_next (cs, next_category);

		if (next_category==first_category || next_category==category)
			break;
	}

	category_store_unlock (cs);
}

void category_store_thread_activate (CategoryStore* cs)
{
	Category* category;
	GtkTreeIter iter;
	gboolean valid;

	category_store_lock (cs);

	valid = gtk_tree_model_get_iter_first (GTK_TREE_MODEL (cs->list_store),
	                                       &iter);
	while (valid) {
		gtk_tree_model_get ( (GtkTreeModel*)cs->list_store,
		                    &iter,
		                    CATEGORY_COL_DATA, &category,
		                    -1);
		category_store_thread_activate_1 (cs, category);
		valid = gtk_tree_model_iter_next ( (GtkTreeModel*)cs->list_store,
		                                  &iter);
	}

	category_store_unlock (cs);
}

void category_store_thread_stop (CategoryStore* cs)
{
	Category* category;
	GtkTreeIter iter;
	gboolean valid;

	valid = gtk_tree_model_get_iter_first (GTK_TREE_MODEL (cs->list_store),
	                                       &iter);
	while (valid) {
		gtk_tree_model_get ( (GtkTreeModel*)cs->list_store,
		                    &iter,
		                    CATEGORY_COL_DATA, &category,
		                    -1);
		category_thread_stop (category);
		valid = gtk_tree_model_iter_next ( (GtkTreeModel*)cs->list_store,
		                                  &iter);
	}
}

void category_store_thread_destroy (CategoryStore* cs)
{
	Category* category;
	GtkTreeIter iter;
	gboolean valid;

	valid = gtk_tree_model_get_iter_first (GTK_TREE_MODEL (cs->list_store),
	                                       &iter);
	while (valid) {
		gtk_tree_model_get ( (GtkTreeModel*)cs->list_store,
		                    &iter,
		                    CATEGORY_COL_DATA, &category,
		                    -1);
		category_thread_destroy (category);
		valid = gtk_tree_model_iter_next ( (GtkTreeModel*)cs->list_store,
		                                  &iter);
	}
}

