#include "urlget.h"
#include "category_view.h"
#include "setting_view_icons.h"

/* data set function for GtkCellRenderer */
static void col_set_icon (GtkTreeViewColumn *tree_column,
                          GtkCellRenderer   *cell,
                          GtkTreeModel      *model,
                          GtkTreeIter       *iter,
                          gpointer           data)
{
	DownloadListView* download_view;
	Category* category;
	guint state;

	gtk_tree_model_get (model, iter,
	                    CATEGORY_COL_DATA, &category,
	                    CATEGORY_COL_DOWNLOAD_VIEW, &download_view,
	                    -1);

	if (&category->completed_view == download_view)
		state = CATEGORY_STATE_COMPLETED;
	else if (&category->recycled_view == download_view)
		state = CATEGORY_STATE_RECYCLED;
	else 
		state = CATEGORY_SETTING(category)->state;

	g_object_set ((GtkCellRenderer*)cell,
	              "pixbuf",
	              download_state_icons[state],
	              NULL);
}

static void col_set_name (GtkTreeViewColumn *tree_column,
                          GtkCellRenderer   *cell,
                          GtkTreeModel      *model,
                          GtkTreeIter       *iter,
                          gpointer           data)
{
	DownloadListView* download_view;
	Category* cate;
	gchar*    name;

	gtk_tree_model_get (model, iter,
	                    CATEGORY_COL_DATA, &cate,
	                    CATEGORY_COL_DOWNLOAD_VIEW, &download_view,
	                    -1);

	if (&cate->completed_view == download_view)
		name = _("Completed");
	else if (&cate->recycled_view == download_view)
		name = _("Recycled");
	else 
		name = cate->setting.name;

	g_object_set ((GtkCellRenderer*)cell,
	              "text",
	              name,
	              NULL);
}

static void col_set_jobs (GtkTreeViewColumn *tree_column,
                          GtkCellRenderer   *cell,
                          GtkTreeModel      *model,
                          GtkTreeIter       *iter,
                          gpointer           data)
{
	gchar* str;
	guint  n_jobs;
	DownloadStore* ds;

	gtk_tree_model_get (model, iter,
	                    CATEGORY_COL_DOWNLOAD_STORE, &ds,
	                    -1);

	n_jobs = gtk_tree_model_iter_n_children ((GtkTreeModel*)ds->list_store,
	                                         NULL);

	str = g_strdup_printf ("%u", n_jobs);
	g_object_set ((GtkCellRenderer*)cell,
	              "text",
	              str,
	              NULL);
	g_free (str);
}

static void category_view_add_column_of_name(GtkTreeView* view)
{
	GtkCellRenderer*   renderer;
	GtkTreeViewColumn* column;

	column = gtk_tree_view_column_new ();
	gtk_tree_view_column_set_title (column, _("Name"));
	renderer = gtk_cell_renderer_pixbuf_new ();
	gtk_tree_view_column_pack_start (column, renderer, FALSE);
	gtk_tree_view_column_set_cell_data_func (column,
	                                         renderer,
	                                         col_set_icon,
	                                         NULL, NULL);
	renderer = gtk_cell_renderer_text_new ();
	gtk_tree_view_column_pack_start (column, renderer, TRUE);
	gtk_tree_view_column_set_cell_data_func (column,
	                                         renderer,
	                                         col_set_name,
	                                         NULL, NULL);
	gtk_tree_view_column_set_resizable (column, TRUE);
	gtk_tree_view_append_column (view, column);
}

GtkTreeView* category_tree_view_new (CategoryStore* cs)
{
	GtkCellRenderer*   renderer;
	GtkTreeViewColumn* column;
	GtkTreeView*       view;

	view = (GtkTreeView*)gtk_tree_view_new ();
	/* render icons for GtkTreeView */
	download_state_icons_render ((GtkWidget*)view);

	gtk_tree_view_set_model (view,
	                         GTK_TREE_MODEL(cs->tree_store));
//	gtk_tree_view_set_rules_hint (view, TRUE);

	/* add columns to GtkTreeView */

	// column Category Name
	category_view_add_column_of_name(view);

	// column jobs
	column = gtk_tree_view_column_new ();
	gtk_tree_view_column_set_title (column, "Jobs");
	renderer = gtk_cell_renderer_text_new ();
	gtk_tree_view_column_pack_start (column, renderer, TRUE);
	gtk_tree_view_column_set_cell_data_func (column,
	                                         renderer,
	                                         col_set_jobs,
	                                         NULL, NULL);
	gtk_tree_view_column_set_resizable (column, TRUE);
	gtk_tree_view_append_column (view, column);

	gtk_widget_show_all ((GtkWidget*)view);

	return view;
}

GtkTreeView* category_list_view_new (CategoryStore* cs)
{
	GtkTreeView*       view;

	view = (GtkTreeView*)gtk_tree_view_new ();
	/* render icons for GtkTreeView */
	download_state_icons_render ((GtkWidget*)view);

	gtk_tree_view_set_model (view,
	                         GTK_TREE_MODEL(cs->list_store));

	/* add columns to GtkTreeView */

	// column Category Name
	category_view_add_column_of_name(view);

	gtk_widget_show_all ((GtkWidget*)view);

	return view;
}

gboolean category_tree_view_get_selected (GtkTreeView*       tree_view,
                                          Category**         category,
                                          DownloadListView** download_view,
                                          DownloadStore**    download_store)
{
	GtkTreeIter       iter;
	GtkTreeModel*     model;
	GtkTreeSelection* selection;
	gboolean          had_selected;

	Category*         cate = NULL;
	DownloadListView* down_view = NULL;
	DownloadStore*    down_store = NULL;

	selection = gtk_tree_view_get_selection (tree_view);
	had_selected = gtk_tree_selection_get_selected (selection,
	                                                &model,
	                                                &iter);
	if (had_selected) {
		gtk_tree_model_get (model,
		                    &iter,
		                    CATEGORY_COL_DATA,
		                    &cate,
		                    CATEGORY_COL_DOWNLOAD_VIEW,
		                    &down_view,
		                    CATEGORY_COL_DOWNLOAD_STORE,
		                    &down_store,
		                    -1);
	}
	if (category)
		*category = cate;
	if (download_view)
		*download_view = down_view;
	if (download_store)
		*download_store = down_store;

	return had_selected;
}

