#include "urlget.h"
#include "download_dialog.h"

DownloadDialog* download_dialog_init (DownloadDialog* dd,
                                      GtkWindow* parent)
{
	GtkWidget* vbox_dlg;

	// set and initial dialog
	dd->parent = parent;
	dd->dialog = (GtkDialog*)gtk_dialog_new_with_buttons(_("Download setting"),
	                                   parent,
			GTK_DIALOG_MODAL | GTK_DIALOG_NO_SEPARATOR | GTK_DIALOG_DESTROY_WITH_PARENT,
	                                   GTK_STOCK_CANCEL,
	                                   GTK_RESPONSE_CANCEL,
	                                   GTK_STOCK_OK,
	                                   GTK_RESPONSE_OK,
	                                   NULL);
	gtk_window_set_resizable (GTK_WINDOW(dd->dialog), FALSE);

	// initial DownloadSettingView
	download_setting_view_init (&dd->setting_view);
	SETTING_VIEW (&dd->setting_view)->parent = GTK_WINDOW(dd->dialog);

	// initial Categorey
	dd->selected_category = NULL;
	dd->category_store = NULL;
	dd->first_select   = TRUE;

	// pack other widget
	dd->hbox = gtk_hbox_new (FALSE, 5);
	vbox_dlg = dd->dialog->vbox;
	gtk_widget_show (dd->hbox);

	gtk_box_pack_end (GTK_BOX(dd->hbox),
	                  dd->setting_view.self,
	                  FALSE, FALSE, 0);
	gtk_box_pack_start (GTK_BOX(vbox_dlg), dd->hbox, FALSE, FALSE, 0);

	return dd;
}

void download_dialog_set_multiple (DownloadDialog* dd,
                                   gboolean multiple)
{
	if (multiple==TRUE) {
		gtk_window_set_title (GTK_WINDOW (dd->dialog),
		                      _("Multiple Download Setting"));
	}

	download_setting_view_set_multiple (&dd->setting_view, multiple);
}

void download_dialog_import_setting (DownloadDialog* dd,
                                     DownloadSetting* ds)
{
	download_setting_view_set (&dd->setting_view, ds);
}

void download_dialog_export_setting (DownloadDialog* dd,
                                     DownloadSetting* ds)
{
	download_setting_view_get (&dd->setting_view, ds);
}

gint  download_dialog_run (DownloadDialog* dd)
{
	gint id;
	gchar* msg;
	gint field_state;
	GtkWidget* dialog;

	while (1) {
		id = gtk_dialog_run (dd->dialog);
		if (id!=GTK_RESPONSE_OK)
			break;

		field_state = download_setting_view_check_field (&dd->setting_view);
		switch (field_state) {
		case DOWNLOAD_SETTING_UNCOMPLETE:
			msg = _("You must complete field : \"Save to\"");
			break;
		case DOWNLOAD_SETTING_UNCOMPLETE_MULTIPlE:
			msg = _("You must complete field : \"URL\" ,\"Save to\" ,and \"Rename\"");
			break;
		case DOWNLOAD_SETTING_UNCOMPLETE_URL:
			msg = _("Not valid URL");
			break;
		case DOWNLOAD_SETTING_COMPLETE:
		default:
			return GTK_RESPONSE_OK;
		}
		// show message
		dialog = gtk_message_dialog_new (dd->parent,
		                                 GTK_DIALOG_DESTROY_WITH_PARENT,
		                                 GTK_MESSAGE_ERROR,
		                                 GTK_BUTTONS_CLOSE,
                                            msg);
		gtk_window_set_transient_for (GTK_WINDOW(dialog),
		                              GTK_WINDOW(dd->dialog));
		gtk_dialog_run (GTK_DIALOG (dialog));
		gtk_widget_destroy (dialog);
	}

	return GTK_RESPONSE_CANCEL;
}

void download_dialog_destroy (DownloadDialog* dd)
{
	gtk_widget_destroy (GTK_WIDGET (dd->dialog));
	g_free (dd);
}

void download_dialog_clear (DownloadDialog* dd)
{
	gtk_widget_destroy (GTK_WIDGET (dd->dialog));
}

// category
static void on_cursor_changed(DownloadDialog* dialog)
{
	GtkTreeSelection* selection;
	GtkTreeModel* model;
	GtkTreeIter iter;
	CategorySetting* setting;

	if (dialog->first_select) {
		download_dialog_select_category (dialog,
		                                 dialog->category_store,
		                                 dialog->selected_category);
		dialog->first_select = FALSE;
		setting = CATEGORY_SETTING (dialog->selected_category);
	}
	else {
		selection = gtk_tree_view_get_selection (dialog->category_view);

		gtk_tree_selection_get_selected (selection,
		                                 &model,
		                                 &iter);
		gtk_tree_model_get (model, &iter,
		                    CATEGORY_COL_DATA, &setting,
		                    -1);
	}
	download_setting_view_set (&dialog->setting_view,
	                           DOWNLOAD_SETTING (setting));

	dialog->selected_category = CATEGORY(setting);
}

void download_dialog_select_category (DownloadDialog* dialog,
                                      CategoryStore* store,
                                      Category* category)
{
	Category* cate_data;
	GtkTreeSelection* selection;
	GtkTreeIter iter;
	gboolean  valid;

	selection = gtk_tree_view_get_selection (dialog->category_view);

	valid = gtk_tree_model_get_iter_first ( (GtkTreeModel*)store->list_store,
	                                       &iter);
    while (valid) {
    	gtk_tree_model_get ( (GtkTreeModel*)store->list_store, &iter,
    	                    CATEGORY_COL_DATA, &cate_data, -1);

		if (category == cate_data) {
			gtk_tree_selection_select_iter (selection,
			                                &iter);
			return;
		}
    	valid = gtk_tree_model_iter_next ( (GtkTreeModel*)store->list_store,
    	                                  &iter);
    }
}

void download_dialog_set_category (DownloadDialog* dd,
                                   CategoryStore* store,
                                   Category* category)
{
	GtkWidget* hbox;
	GtkWidget* widget;
	GtkWidget* vbox;
	GtkScrolledWindow* scrolled;

	// initial
	dd->category_store = store;
	dd->selected_category = category;

	// set and initial category
	vbox = gtk_vbox_new (FALSE, 3);
	widget = gtk_label_new (_("Select Category:"));

	dd->category_view = category_list_view_new (store);
	gtk_widget_set_size_request ( (GtkWidget*)dd->category_view, 120, 150);
	gtk_tree_view_set_headers_visible (dd->category_view, FALSE);
	gtk_container_set_border_width (GTK_CONTAINER(dd->category_view), 5);

	// scrolled window for category_view
	scrolled = (GtkScrolledWindow*)gtk_scrolled_window_new (NULL, NULL);
	gtk_scrolled_window_set_policy (scrolled,
	                                GTK_POLICY_AUTOMATIC,
	                                GTK_POLICY_AUTOMATIC);
	gtk_scrolled_window_set_shadow_type (scrolled,
	                                     GTK_SHADOW_IN);
	gtk_container_add (GTK_CONTAINER (scrolled),
	                   GTK_WIDGET (dd->category_view));

	gtk_box_pack_start (GTK_BOX(vbox),
	                    widget, FALSE, FALSE, 0);
	gtk_box_pack_start (GTK_BOX(vbox),
	                    (GtkWidget*)scrolled,
	                    TRUE, TRUE, 0);
	gtk_widget_show_all (vbox);

	// pack category view to dialog
	hbox = dd->hbox;
	widget = gtk_vseparator_new ();
	gtk_widget_show (widget);
	gtk_box_pack_end (GTK_BOX(hbox),
	                    widget,
	                    FALSE, FALSE, 0);
	gtk_box_pack_end (GTK_BOX(hbox),
	                  vbox,
	                  TRUE, TRUE, 0);

	// connect signal to handler
	g_signal_connect_swapped (dd->category_view, "cursor-changed",
	                          G_CALLBACK(on_cursor_changed), dd);
	/*
	if (category)
		download_dialog_select_category (dd, store, category);
	*/
}

void download_dialog_set_global_setting (DownloadDialog* dd,
                                         GlobalSetting* gs)
{
	download_setting_view_set_history (&dd->setting_view,
	                                   gs->directory_list);
}

