/*  download_setting.h
 *
 *
 *  2003  Plus Huang
 */

#ifndef __DOWNLOAD_SETTING_H
#define __DOWNLOAD_SETTING_H

#include "proxy_setting.h"

#ifdef __cplusplus
extern "C" {
#endif

#define DOWNLOAD_SETTING(o)   ((DownloadSetting*)(o))
#define DOWNLOAD_SETTING_TAG  "download_setting"

typedef struct _DownloadSetting   DownloadSetting;

struct _DownloadSetting {
	ProxySetting  proxy;

	unsigned int  n_parts;
	unsigned int  n_redirect;
	unsigned int  n_retries;
	unsigned int  retry_delay;

	unsigned int  multiple:1;         // if TRUE, filename and url will be ignored
	unsigned int  start_mode:3;
	unsigned int  state:4;            // download runtime state
	char* url;
	char* directory;
	char* filename;
};

enum DOWNLOAD_SETTING_START_MODE {
	DOWNLOAD_START_AUTO,
	DOWNLOAD_START_MANUAL,
};

enum DOWNLOAD_STATE {
	DOWNLOAD_STATE_WAITING,
	DOWNLOAD_STATE_PAUSE,
	DOWNLOAD_STATE_ERROR,
	DOWNLOAD_STATE_EXECUTING,
	DOWNLOAD_STATE_RETRY,
	DOWNLOAD_STATE_COMPLETED,
	DOWNLOAD_STATE_RECYCLED,
	DOWNLOAD_N_STATE
};

DownloadSetting* download_setting_init (DownloadSetting* dd);
void download_setting_destroy (DownloadSetting* dd);
void download_setting_clear (DownloadSetting* dd);
void download_setting_assign (DownloadSetting* dd, DownloadSetting* src);
void download_setting_set_start_mode (DownloadSetting* dd, int mode);

#define download_setting_new()         \
        download_setting_init (g_malloc (sizeof(DownloadSetting)))

#define download_setting_set_url(d, str)          setting_set_string (&(d)->url, str)
#define download_setting_set_directory(d, str)    setting_set_string (&(d)->directory, str)
#define download_setting_set_filename(d, str)     setting_set_string (&(d)->filename, str)

void download_setting_parse_xml_attr (DownloadSetting* setting,
                                      const char** attr_names,
                                      const char** attr_values);
void download_setting_write_xml (DownloadSetting* setting,
                                 FILE* file, int level);

#ifdef __cplusplus
}
#endif

#endif  // End of __DOWNLOAD_SETTING_H
