//
//  download_store.h
//
//    store DownloadSetting and DownloadStatus in GtkListStore
//
//  2003   Plus Huang
//

#ifndef __DOWNLOAD_STORE_H
#define __DOWNLOAD_STORE_H

#include <glib.h>
#include <gtk/gtk.h>
#include "download.h"

#ifdef __cplusplus
extern "C" {
#endif

typedef struct _DownloadStore     DownloadStore;

struct _DownloadStore {
	GtkListStore*    list_store;

	guint            capacity_limit;
	gboolean         historical;
	// for thread
	GStaticRecMutex  mutex;
};

// columns for GtkListStore
enum DOWNLOAD_COLUMN_NAME {
	DOWNLOAD_COL_DATA,
	DOWNLOAD_COL_THREAD,
	DOWNLOAD_N_COL
};


// init and clear
void download_store_init (DownloadStore* ds);
void download_store_clear (DownloadStore* ds);

// lock and unlock functions
#define download_store_lock(ds)      g_static_rec_mutex_lock (&(ds)->mutex)
#define download_store_unlock(ds)    g_static_rec_mutex_unlock (&(ds)->mutex)

// data operate functions
void download_store_add (DownloadStore* ds, Download* dl);
void download_store_add_static (DownloadStore* ds, Download* dl);

void download_store_erase (DownloadStore* ds, GtkTreeIter* iter);
void download_store_erase_from_nth (DownloadStore* ds, guint nth);
void download_store_erase_excess (DownloadStore* ds);

void download_store_move_to_store (DownloadStore* ds_src,
                                   DownloadStore* ds_dest,
                                   GtkTreeIter* iter_src);

gboolean download_store_find_waiting (DownloadStore* ds,
                                      GtkTreeIter*   iter);

void download_store_set_thread(DownloadStore*  ds,
                               GtkTreeIter*    iter,
                               gpointer        thread);

void download_store_row_changed (DownloadStore* ds,
                                 GtkTreeIter* iter);

// void download_store_get_data(DownloadStore*    ds,
//                              GtkTreeIter*      iter,
//                              Download**        data);
#define download_store_get_data(ds, iter, data)   \
        gtk_tree_model_get ( (GtkTreeModel*) (ds)->list_store, iter,    \
                            DOWNLOAD_COL_DATA, data, -1)

#define download_store_set_historical(ds, b)  (ds)->historical=(b)

#ifdef __cplusplus
}
#endif

#endif  // End of __DOWNLOAD_STORE_H

