#include <string.h>
#include <stdlib.h>
#include "global_setting.h"
#include "urlget.h"


GlobalSetting* global_setting_new ()
{
	GlobalSetting* gs = g_malloc (sizeof (GlobalSetting));

	global_setting_init (gs);

	return gs;
}

void global_setting_destroy (GlobalSetting* gs)
{
	global_setting_clear (gs);
	g_free (gs);
}

void global_setting_init (GlobalSetting* gs)
{
	category_setting_init (&gs->category);

	gs->category.multiple = 1;  // TRUE
	gs->n_active_category = 2;
	gs->header_category_visible = 1;

	gs->directory_list = NULL;
	gs->n_directory = 0;
}

void global_setting_clear (GlobalSetting* gs)
{
	category_setting_clear (&gs->category);
}

void global_setting_assign (GlobalSetting* gs, GlobalSetting* src)
{
	category_setting_assign (&gs->category, &src->category);

	gs->n_active_category = src->n_active_category;
}

void   global_setting_add_directory (GlobalSetting* gs,
                                     const gchar* directory)
{
	GList* node;
	gchar* string;

	if (directory==NULL || *directory==0)
		return;

	node = g_list_find_custom (gs->directory_list,
	                           directory,
	                           (GCompareFunc)strcmp);

	if (node) {
		string = node->data;
		gs->directory_list = g_list_delete_link (gs->directory_list, node);
		gs->n_directory--;
	}
	else {
		if (gs->n_directory > 8) {
			if (node = g_list_last (gs->directory_list))
				g_free (node->data);
			gs->directory_list = g_list_delete_link (gs->directory_list, node);
			gs->n_directory--;
		}
		string = g_strdup (directory);
	}

	gs->directory_list = g_list_prepend (gs->directory_list, string);
	gs->n_directory++;
}

GList* global_setting_get_directory_list (GlobalSetting* gs)
{
	return gs->directory_list;
}

void global_setting_parse_xml_attr (GlobalSetting* setting,
                                    const char** attr_names,
                                    const char** attr_values)
{
	int ii;

	for (ii=0; attr_names[ii]; ii++) {
		if (strcmp (attr_names[ii], "n_active_category")==0)
			setting->n_active_category = atoi (attr_values[ii]);
		if (strcmp (attr_names[ii], "header_category_visible")==0)
			setting->header_category_visible = atoi (attr_values[ii]);
	}
}

void global_setting_write_xml (GlobalSetting* setting,
                               FILE* file, int level)
{
	GList* list = g_list_last (setting->directory_list);

	if (level)
		fprintf (file, "%*c", level, ' ');

	fprintf (file,
	         "<" GLOBAL_SETTING_TAG " "       // Start of format string
	         "n_active_category=\"%d\" "
	         "header_category_visible=\"%d\">"
	         "\n",                            // End of format string
	         setting->n_active_category,
	         setting->header_category_visible
	         );
	category_setting_write_xml (CATEGORY_SETTING (setting), file, level+1);

	// write directory
	for ( ; list; list=list->prev) {
		fprintf (file,
		         "%*c"
		         "<directory>%s</directory>\n",
				 level, ' ',
		         list->data);
	}

	if (level)
		fprintf (file, "%*c", level, ' ');

	fputs ("</" GLOBAL_SETTING_TAG ">\n", file);
}

