#include <stdio.h>
#include <gdk/gdkkeysyms.h>
#include "urlget.h"
#include "utility.h"
#include "main_app.h"
#include "download_dialog.h"
#include "category_dialog.h"
#include "batch_dialog.h"
#include "download_thread.h"
#include "global_dialog.h"
#include "category_selection_dialog.h"

// utility functions ------------------------------------------------

static void add_batch_download (Category* cate,
                                Download* download_data,
                                const gchar* pattern,
                                guint n_min,
                                guint n_max,
                                guint n_width)
{
	DownloadSetting* setting = DOWNLOAD_SETTING (download_data);
	const gchar* p_star;
	GString *str_url;
	gssize   truncate_len;
	guint    n_serial;
	UrlPart  up;

	for (p_star=pattern; *p_star; ++p_star) {
		if (*p_star=='*')
			break;
	}
	if (*p_star == 0)
		return;


	str_url = g_string_new_len (pattern, p_star-pattern);
	truncate_len = str_url->len;
	g_free (setting->filename);
	g_free (setting->url);

	for (n_serial=n_min; n_serial<=n_max; ++n_serial) {
		if (n_width) {
			g_string_append_printf (str_url,
			                        "%.*u%s",
			                        n_width,
			                        n_serial,
			                        p_star+1);
		}
		else {  // character mode
			g_string_append_printf (str_url,
			                        "%c%s",
			                        (unsigned char)n_serial,
			                        p_star+1);
		}
		setting->url = str_url->str;
		// get filename
		url_part (&up, setting->url);
		setting->filename = up.name_beg;

		download_store_add (&cate->waiting_store, download_data);
		g_string_truncate (str_url, truncate_len);
	}
	g_string_free (str_url, TRUE);

	setting->filename = NULL;
	setting->url      = NULL;
}

// utility function for GtkTreeSelection
// malloc and return Selected GtkTreeIter
// return value : number of Selected GtkTreeIter
guint tree_selection_get_selected_iters (GtkTreeSelection* selection,
                                         GtkTreeIter** iter_array)
{
	GtkTreeModel*     model;
	GtkTreeIter*      p_iter;
	GList*            path_list;
	GList*            node_prev;
	guint             n_iter;

	if (n_iter = gtk_tree_selection_count_selected_rows (selection)) {
		// allocate array of iter
		*iter_array = g_malloc (sizeof (GtkTreeIter) * n_iter);
		// get GList contain GtkTreePath
		path_list = gtk_tree_selection_get_selected_rows (selection, &model);
		// convert GtkTreePath to GtkTreeIter
		p_iter = *iter_array;
		while (path_list) {
			gtk_tree_model_get_iter (model, p_iter, path_list->data);
            p_iter++;
			node_prev = path_list;
			path_list = path_list->next;
			gtk_tree_path_free (node_prev->data);
			g_list_free_1 (node_prev);
		}
		return n_iter;
	}
	return 0;
}

static void show_about_dialog (GtkWindow* parent)
{
	GtkWidget* vbox;
//	GtkWidget* image;
	GtkWidget* label;
	GtkWidget* dialog;

	dialog = gtk_dialog_new_with_buttons (_("About UrlGet"),
	                                      parent,
	         (GTK_DIALOG_MODAL | GTK_DIALOG_NO_SEPARATOR | GTK_DIALOG_DESTROY_WITH_PARENT),
	                                      GTK_STOCK_OK,
	                                      GTK_RESPONSE_OK,
	                                      NULL);
	gtk_window_set_default_size (GTK_WINDOW (dialog), 150, 150);
	vbox = GTK_DIALOG (dialog)->vbox;
	// icon
//	image = gtk_image_new_from_file (PACKAGE_PIXMAPS_DIR G_DIR_SEPARATOR_S"urlget_icon.png");
//	gtk_box_pack_start (GTK_BOX(vbox), image, FALSE, FALSE, 0);
	label = gtk_label_new ("UrlGet 0.7 beta");
	gtk_box_pack_start (GTK_BOX(vbox), label, TRUE, TRUE, 0);
	label = gtk_label_new (_("Copyright(C) 2002-2004 Plus Huang"));
	gtk_box_pack_start (GTK_BOX(vbox), label, TRUE, TRUE, 0);
	label = gtk_label_new ("http://urlget.sourceforge.net/");
	gtk_box_pack_start (GTK_BOX(vbox), label, TRUE, TRUE, 0);
	gtk_widget_show_all (vbox);

	gtk_dialog_run (GTK_DIALOG (dialog));
	gtk_widget_destroy (dialog);
}

// End of utility functions =========================================


// download signal handler ------------------------------------------
static void on_download_start (MainApp* app)
{
	GtkTreeSelection* selection;
	DownloadListView* download_view;
	DownloadStore*    download_store;
	Category*         category;

	DownloadSetting*  setting;
	DownloadThread*   thread;

	GtkTreeIter*      iter_array;
	guint             n_iter;
	guint             index;

	gboolean          need_active = FALSE;

	if (category_tree_view_get_selected (app->window.category_view,
                                         &category,
                                         &download_view,
                                         &download_store)==FALSE )
	{
		return;
	}

	selection = gtk_tree_view_get_selection (download_view->self);

	download_store_lock (download_store);

	if ( n_iter=tree_selection_get_selected_iters (selection, &iter_array) )
	{
		for (index=0; index<n_iter; index++) {
			gtk_tree_model_get ((GtkTreeModel*)download_store->list_store,
	                            iter_array + index,
			                    DOWNLOAD_COL_DATA,   &setting,
			                    DOWNLOAD_COL_THREAD, &thread,
			                    -1);
			if (thread==NULL) {
				setting->state = DOWNLOAD_STATE_WAITING;
				download_store_row_changed (download_store, iter_array+index);
			}
		}
		g_free (iter_array);
		need_active = TRUE;
	}

	download_store_unlock (download_store);

	if (need_active)
		category_store_thread_activate_1 (&app->category_store, category);
}

static void on_download_stop (MainApp* app)
{
	GtkTreeSelection* selection;
	DownloadListView* download_view;
	DownloadStore*    download_store;
	Category*         category;

	DownloadSetting*  setting;
	DownloadThread*   thread;

	GtkTreeIter*      iter_array;
	guint             n_iter;
	guint             index;

	if (category_tree_view_get_selected (app->window.category_view,
                                         &category,
                                         &download_view,
                                         &download_store)==FALSE )
	{
		return;
	}

	selection = gtk_tree_view_get_selection (download_view->self);

	download_store_lock (download_store);

	if ( n_iter=tree_selection_get_selected_iters (selection, &iter_array) )
	{
		for (index=0; index<n_iter; index++) {
			gtk_tree_model_get ((GtkTreeModel*)download_store->list_store,
	                            iter_array + index,
			                    DOWNLOAD_COL_DATA,   &setting,
			                    DOWNLOAD_COL_THREAD, &thread,
			                    -1);
			if (thread)
				download_thread_skip (thread);
			else {
				setting->state = DOWNLOAD_STATE_PAUSE;
				download_store_row_changed (download_store, iter_array+index);
			}
		}
		g_free (iter_array);
	}

	download_store_unlock (download_store);
}

static void on_download_new (MainApp* app)
{
	GtkClipboard*  clipboard;
	gchar*         clip_text = NULL;

	Category* selected_category = NULL;
	Download* download_data;
	UrlPart   urlp;
	gboolean  had_selected;

	// check Clipboard
	clipboard = gtk_clipboard_get (GDK_SELECTION_CLIPBOARD);
	if ( gtk_clipboard_wait_is_text_available (clipboard) )
		clip_text = gtk_clipboard_wait_for_text (clipboard);

	// add download from url
	download_data = download_new ();
	url_part (&urlp, clip_text);
	if ( urlp.protocol_beg!=NULL )
		download_setting_set_url (DOWNLOAD_SETTING (download_data), clip_text);

	had_selected = main_app_get_download_from_user (app,
	                                                DOWNLOAD_SETTING (download_data),
	                                                &selected_category,
	                                                FALSE);
	if (had_selected) {
		download_store_add_static (&selected_category->waiting_store,
		                           download_data);
		category_store_thread_activate_1 (&app->category_store,
		                                  selected_category);
	}
	else
		download_unref (download_data);

	g_free (clip_text);
}

static void on_download_batch (MainApp* app)
{
	BatchDialog bat_dialog;
	gint        batch_mode;
	gint        max, min, width;
	gchar*      pattern;

	Download*   download_data;
	gboolean    had_selected;
	Category*   selected_category;

	// batch dialog
	batch_dialog_init (&bat_dialog, app->window.self);

	if ( (batch_mode=batch_dialog_run (&bat_dialog))==BATCH_MODE_NONE ) {
		batch_dialog_clear (&bat_dialog);
		return;
	}
	if (batch_mode == BATCH_MODE_CHAR) {
		max = batch_dialog_get_char_max (&bat_dialog);
		min = batch_dialog_get_char_min (&bat_dialog);
		width = 0;
	}
	else {
		max = batch_dialog_get_int_max (&bat_dialog);
		min = batch_dialog_get_int_min (&bat_dialog);
		width = batch_dialog_get_int_width (&bat_dialog);
	}
	pattern  = g_strdup (batch_dialog_get_pattern (&bat_dialog) );

	batch_dialog_clear (&bat_dialog);

	// download import dialog
	// initial
	download_data = download_new ();
	had_selected = main_app_get_download_from_user (app,
	                                                DOWNLOAD_SETTING (download_data),
	                                                &selected_category,
	                                                TRUE);
	if (had_selected) {
		add_batch_download (selected_category,
		                    download_data,
		                    pattern,
		                    min,
		                    max,
		                    width);
		// try to start-up thread.
		category_store_thread_activate_1 (&app->category_store,
		                                  selected_category);
	}

	download_unref (download_data);
	g_free (pattern);
}

// bugs here
static void on_download_erase (MainApp* app)
{
	GdkWindow*        gdk_win;
	GdkModifierType   mask;

	GtkTreeSelection* selection;
	DownloadListView* download_view;
	DownloadStore*    download_store;
	Category*         category;

	Download*         dl_data;
	DownloadThread*   thread;

	GtkTreeIter*      iter_array;
	guint             n_iter;
	guint             index;

	if (category_tree_view_get_selected (app->window.category_view,
                                         &category,
                                         &download_view,
                                         &download_store)==FALSE )
	{
		return;
	}

	selection = gtk_tree_view_get_selection (download_view->self);

	download_store_lock (download_store);

	if ( n_iter=tree_selection_get_selected_iters (selection, &iter_array) )
	{
		gdk_win = gtk_widget_get_parent_window ((GtkWidget*)download_view->self);
		gdk_window_get_pointer (gdk_win, NULL, NULL, &mask);

		for (index=0; index<n_iter; index++) {
			gtk_tree_model_get ((GtkTreeModel*)download_store->list_store,
	                            iter_array + index,
			                    DOWNLOAD_COL_DATA, &dl_data,
			                    DOWNLOAD_COL_THREAD, &thread,
			                    -1);
			if (thread)
				download_thread_job_erased (thread);
			if ( &category->recycled_store == download_store || mask & GDK_SHIFT_MASK)
				download_store_erase (download_store, iter_array+index);
			else {
				download_store_move_to_store (download_store,
				                              &category->recycled_store,
				                              iter_array + index);
			}
		}
		g_free (iter_array);

		category_changed (category);
	}

	download_store_unlock (download_store);
}

static void on_download_property (MainApp* app)
{
	DownloadListView* download_view;
	DownloadStore*    download_store;
	Download*         dl_data;
	GList*            dl_list;
	GList*            node;
	gint              n_iter;

	if (category_tree_view_get_selected (app->window.category_view,
                                         NULL,
                                         &download_view,
                                         &download_store)==FALSE )
	{
		return;
	}

	dl_list = download_list_view_selected_list_copy (download_view, &n_iter);

	if (n_iter) {
		dl_data = download_new ();
		download_assign (dl_data, dl_list->data);
		if (n_iter>1)
			DOWNLOAD_SETTING (dl_data)->multiple = TRUE;
		if ( main_app_get_download_from_user (app, DOWNLOAD_SETTING (dl_data),
		                                      NULL, (n_iter>1) ? TRUE : FALSE) )
		{
			for (node=dl_list; node; node=node->next)
				download_assign (node->data, dl_data);
		}
		download_unref (dl_data);
	}

	download_list_view_selected_list_free (dl_list);

	main_window_text_view_update (&app->window);
}

static void on_download_move_to (MainApp* app)
{
	CategorySelectionDialog* csd;
	DownloadListView* dlv_src;
	DownloadListView* dlv_dest;
	gboolean  had_selected;

	had_selected = category_tree_view_get_selected (app->window.category_view,
		                                            NULL,
	                                                &dlv_src,
		                                            NULL );

	csd = category_selection_dialog_new (_("Move selected download to..."),
	                                     &app->category_store,
	                                     app->window.self);

	if (gtk_dialog_run (csd->self)==GTK_RESPONSE_OK) {
		had_selected = category_tree_view_get_selected (csd->category_view,
		                                                NULL,
		                                                &dlv_dest,
		                                                NULL);
		if (had_selected && dlv_src!=dlv_dest) {
			main_app_move_selected_download (app, dlv_src, dlv_dest);
			main_window_text_view_update (&app->window);
		}
	}

	category_selection_dialog_destroy (csd);
}

static void on_download_move_top (MainApp* app)
{
	GtkTreeSelection* selection;
	DownloadListView* download_view;
	DownloadStore*    download_store;
	GtkTreeIter*      iter_array;
	guint             n_iter;

	if (category_tree_view_get_selected (app->window.category_view,
                                         NULL,
                                         &download_view,
                                         &download_store)==FALSE )
	{
		return;
	}

	selection = gtk_tree_view_get_selection (download_view->self);

	download_store_lock (download_store);

	if ( n_iter=tree_selection_get_selected_iters (selection, &iter_array) )
	{
		while (n_iter--) {
			gtk_list_store_move_after (download_store->list_store,
			                           iter_array + n_iter, NULL);
		}
		g_free (iter_array);
	}

	download_store_unlock (download_store);
}

static void on_download_move_bottom (MainApp* app)
{
	GtkTreeSelection* selection;
	DownloadListView* download_view;
	DownloadStore*    download_store;
	GtkTreeIter*      iter_array;
	guint             n_iter;
	guint             index;

	if (category_tree_view_get_selected (app->window.category_view,
                                         NULL,
                                         &download_view,
                                         &download_store)==FALSE )
	{
		return;
	}

	selection = gtk_tree_view_get_selection (download_view->self);

	download_store_lock (download_store);

	if ( n_iter=tree_selection_get_selected_iters (selection, &iter_array) )
	{
		for (index=0; index<n_iter; index++) {
			gtk_list_store_move_before (download_store->list_store,
			                            iter_array + index, NULL);
		}
		g_free (iter_array);
	}

	download_store_unlock (download_store);
}

// End of download signal handler ===================================


// category signal handler ------------------------------------------

static void on_category_new (MainApp* app)
{
	CategoryDialog* dialog;
	CategorySetting setting;
	Category* category;

	dialog = category_dialog_new (app->window.self);
	category_setting_init (&setting);
	download_setting_view_set_history (DOWNLOAD_SETTING_VIEW (&dialog->setting_view),
	                                   app->setting->directory_list);

	category_setting_view_set (&dialog->setting_view, &setting);

	if (gtk_dialog_run (GTK_DIALOG (dialog->self))==GTK_RESPONSE_OK) {
		category_setting_view_get (&dialog->setting_view, &setting);
		category = category_new_from_setting (&setting);
		category_store_append (&app->category_store, category);
		main_app_connect_category_signal (app, category);
	}

	category_setting_clear (&setting);
	category_dialog_destroy (dialog);
}

static void on_category_erase (MainApp* app)
{
	Category*   category;
	GtkWidget*  dialog;
	gint        response;

	if ( category_tree_view_get_selected (app->window.category_view,
	                                      &category, NULL, NULL)==FALSE )
	{
		return;
	}

	dialog = gtk_message_dialog_new (app->window.self,
	                                 GTK_DIALOG_DESTROY_WITH_PARENT,
	                                 GTK_MESSAGE_WARNING,
	                                 GTK_BUTTONS_YES_NO,
	   _("This will erase category and it's download. Are you sure to erase?"));
	response = gtk_dialog_run (GTK_DIALOG(dialog));
	gtk_widget_destroy(dialog);

	if (response == GTK_RESPONSE_YES) {
		main_window_on_category_erasing (&app->window, category);
	//	category_thread_destroy (category);
		category_store_erase (&app->category_store, category);

		// check selection when erase category
		main_window_selection_changed (&app->window);
	}
}

static void on_category_property (MainApp* app)
{
	CategoryDialog* dialog;
	Category*       category;

	if ( category_tree_view_get_selected (app->window.category_view,
	                                      &category, NULL, NULL)==FALSE )
	{
		return;
	}

	dialog = category_dialog_new (app->window.self);
	download_setting_view_set_history (DOWNLOAD_SETTING_VIEW (&dialog->setting_view),
	                                   app->setting->directory_list);

	category_setting_view_set (&dialog->setting_view,
	                           CATEGORY_SETTING(category));

	if (gtk_dialog_run (GTK_DIALOG (dialog->self))==GTK_RESPONSE_OK) {
		category_setting_view_get (&dialog->setting_view,
		                           CATEGORY_SETTING(category));
		category_apply_setting (category);
	}

	category_dialog_destroy (dialog);
}

// End of category signal handler ===================================


// global signal handler --------------------------------------------

static void on_set_category_default (MainApp* app)
{
	CategoryDialog* dialog;

	dialog = category_dialog_new (app->window.self);
	download_setting_view_set_history (DOWNLOAD_SETTING_VIEW (&dialog->setting_view),
	                                   app->setting->directory_list);

	category_setting_view_set (&dialog->setting_view,
	                           CATEGORY_SETTING(app->setting));
    category_setting_view_set_multiple (&dialog->setting_view, TRUE);

	if (gtk_dialog_run (GTK_DIALOG (dialog->self))==GTK_RESPONSE_OK) {
		category_setting_view_get (&dialog->setting_view,
		                           CATEGORY_SETTING(app->setting));
	}

	category_dialog_destroy (dialog);
}

static void on_set_global_setting (MainApp* app)
{
	GlobalDialog* gd;

	gd = global_dialog_new (app->window.self);

	global_dialog_set (gd, app->setting);

	if (gtk_dialog_run (gd->self)==GTK_RESPONSE_OK)
		global_dialog_get (gd, app->setting);

	global_dialog_destroy (gd);

	main_app_apply_setting (app);
	category_store_thread_activate (&app->category_store);
}
// End of global signal handler =====================================

// keyboard
gboolean on_key_press_event(MainApp* app, 
                            GdkEventKey *event,
                            GtkWidget *widget)
{
	if (event->keyval==GDK_Delete) {
		if (widget == (GtkWidget*)app->window.category_view)
			on_category_erase (app);
		else
			on_download_erase (app);
	}
	return FALSE;
}

// MainApp
void main_app_init (MainApp* app)
{
	const gchar* home_dir = g_get_home_dir ();

	category_store_init (&app->category_store);
	main_window_init (&app->window, &app->category_store);

	// read setting file
	if (main_app_read_setting_file (app, urlget_get_conf_filename ())==FALSE)
		app->setting = global_setting_new ();

	// read data file
	if (main_app_read_data_file (app, urlget_get_data_filename ())==FALSE) {
		Category* category = category_new();

		category_setting_set_name (CATEGORY_SETTING (category), "Home");
		download_setting_set_directory (DOWNLOAD_SETTING (category), home_dir);

		category_store_append (&app->category_store, category);
		gtk_tree_view_expand_all (app->window.category_view);
	}

	main_app_apply_setting (app);

	// window
	g_signal_connect (app->window.self, "destroy",
	                  G_CALLBACK(gtk_main_quit), NULL);
	g_signal_connect_swapped (app->window.self, "delete-event",
	                          G_CALLBACK(main_app_on_delete_event), app);
	g_signal_connect_swapped (app->window.category_view, "key-press-event",
                              G_CALLBACK (on_key_press_event), app);


	// file menu
	g_signal_connect_swapped (app->window.file_menu.load, "activate",
	                          G_CALLBACK(main_app_on_load), app);
	g_signal_connect_swapped (app->window.file_menu.save, "activate",
	                          G_CALLBACK(main_app_on_save), app);
	g_signal_connect_swapped (app->window.file_menu.save_as, "activate",
	                          G_CALLBACK(main_app_on_save_as), app);
	g_signal_connect_swapped (app->window.file_menu.import_html, "activate",
	                          G_CALLBACK(main_app_on_import_html), app);
	g_signal_connect_swapped (app->window.file_menu.export, "activate",
	                          G_CALLBACK(main_app_on_export_url), app);

	// download
	g_signal_connect_swapped (app->window.button_start, "clicked",
	                          G_CALLBACK(on_download_start), app);
	g_signal_connect_swapped (app->window.button_stop, "clicked",
	                          G_CALLBACK(on_download_stop), app);
	g_signal_connect_swapped (app->window.button_new, "clicked",
	                          G_CALLBACK(on_download_new), app);
	g_signal_connect_swapped (app->window.button_batch, "clicked",
	                          G_CALLBACK(on_download_batch), app);
	g_signal_connect_swapped (app->window.button_erase, "clicked",
	                          G_CALLBACK(on_download_erase), app);
	g_signal_connect_swapped (app->window.button_property, "clicked",
	                          G_CALLBACK(on_download_property), app);
	g_signal_connect_swapped (app->window.button_move_top, "clicked",
	                          G_CALLBACK(on_download_move_top), app);
	g_signal_connect_swapped (app->window.button_move_bottom, "clicked",
	                          G_CALLBACK(on_download_move_bottom), app);
	// CategoryMenu
	g_signal_connect_swapped (app->window.category_menu.new_category, "activate",
	                          G_CALLBACK(on_category_new), app);
	g_signal_connect_swapped (app->window.category_menu.erase_category, "activate",
	                          G_CALLBACK(on_category_erase), app);
	g_signal_connect_swapped (app->window.category_menu.properties, "activate",
	                          G_CALLBACK(on_category_property), app);
	// DownloadMenu
	g_signal_connect_swapped (app->window.download_menu.start, "activate",
	                          G_CALLBACK(on_download_start), app);
	g_signal_connect_swapped (app->window.download_menu.stop, "activate",
	                          G_CALLBACK(on_download_stop), app);
	g_signal_connect_swapped (app->window.download_menu.new_download, "activate",
	                          G_CALLBACK(on_download_new), app);
	g_signal_connect_swapped (app->window.download_menu.add_batch, "activate",
	                          G_CALLBACK(on_download_batch), app);
	g_signal_connect_swapped (app->window.download_menu.erase, "activate",
	                          G_CALLBACK(on_download_erase), app);
	g_signal_connect_swapped (app->window.download_menu.properties, "activate",
	                          G_CALLBACK(on_download_property), app);
	g_signal_connect_swapped (app->window.download_menu.move_to, "activate",
	                          G_CALLBACK(on_download_move_to), app);
	g_signal_connect_swapped (app->window.download_menu.move_top, "activate",
	                          G_CALLBACK(on_download_move_top), app);
	g_signal_connect_swapped (app->window.download_menu.move_bottom, "activate",
	                          G_CALLBACK(on_download_move_bottom), app);
	// Global Menu
	g_signal_connect_swapped (app->window.global_menu.start_job, "activate",
	                          G_CALLBACK(category_store_thread_activate),
	                          &app->category_store);
	g_signal_connect_swapped (app->window.global_menu.stop_all, "activate",
	                          G_CALLBACK(category_store_thread_stop),
	                          &app->category_store);
	g_signal_connect_swapped (app->window.global_menu.category_default, "activate",
	                          G_CALLBACK(on_set_category_default), app);
	g_signal_connect_swapped (app->window.global_menu.setting, "activate",
	                          G_CALLBACK(on_set_global_setting), app);

	// Help Menu
	g_signal_connect_swapped (app->window.help_menu.about_urlget, "activate",
	                          G_CALLBACK(show_about_dialog), app->window.self);

	main_window_selection_changed (&app->window);

	// init Drag and Drop
	main_app_dnd_init (app);
}

void main_app_run (MainApp* app)
{
	gtk_widget_show (GTK_WIDGET (app->window.self));
}

gboolean main_app_on_delete_event (MainApp* app)
{
	GtkWidget* dialog;
	gint       response;

	if (category_store_thread_is_active (&app->category_store)) {
		dialog = gtk_message_dialog_new (app->window.self,
		                                 GTK_DIALOG_DESTROY_WITH_PARENT,
		                                 GTK_MESSAGE_WARNING,
		                                 GTK_BUTTONS_YES_NO,
		             _("There are still downloading. Are you sure to quit?"));
		response = gtk_dialog_run (GTK_DIALOG(dialog));
		gtk_widget_destroy (dialog);

		if (response==GTK_RESPONSE_NO)
			return TRUE;
	}
	category_store_thread_destroy (&app->category_store);
	main_app_on_save (app);

	return FALSE;
}

void main_app_apply_setting (MainApp* app)
{
	app->category_store.n_thread_limit = app->setting->n_active_category;
	gtk_tree_view_set_headers_visible (app->window.category_view,
	                                   app->setting->header_category_visible);
}

gboolean main_app_get_download_from_user (MainApp* app,
                                          DownloadSetting* setting,
                                          Category** selected_category,
                                          gboolean multiple)
{
	DownloadDialog* dd = download_dialog_new (app->window.self);
	Category*  category;
	gboolean   had_input = FALSE;


	if (selected_category) {
		category_tree_view_get_selected (app->window.category_view,
		                                 &category, NULL, NULL);
		download_dialog_set_category (dd,
		                              &app->category_store,
		                              category);
	}

	download_setting_view_set_history (&dd->setting_view,
	                                   app->setting->directory_list);
	download_dialog_import_setting (dd, setting);
	download_dialog_set_multiple (dd, multiple);

	if (download_dialog_run (dd)==GTK_RESPONSE_OK) {
		download_dialog_export_setting (dd, setting);
		global_setting_add_directory (app->setting, setting->directory);
		had_input = TRUE;

		if (selected_category)
			*selected_category = dd->selected_category;
	}

	download_dialog_destroy (dd);

	return had_input;
}

void main_app_move_selected_download (MainApp* app,
                                      DownloadListView* dlv_src,
                                      DownloadListView* dlv_dest)
{
	guint n_iter, index;
	GtkTreeIter* iter_array;
	GtkTreeSelection* selection;
	DownloadStore* ds_src  = dlv_src->store;
	DownloadStore* ds_dest = dlv_dest->store;

	selection = gtk_tree_view_get_selection (dlv_src->self);

	download_store_lock (ds_src);

	n_iter = tree_selection_get_selected_iters (selection, &iter_array);

	for (index=0; index < n_iter; index++) {
		download_store_move_to_store (ds_src,
		                              ds_dest,
		                              iter_array+index);
	}
	g_free (iter_array);

	download_store_unlock (ds_src);

	// update CategoryTree
	gtk_widget_queue_draw (GTK_WIDGET (app->window.category_view));
}

void main_app_connect_category_signal (MainApp* app,
                                       Category* category)
{
	main_window_connect_download_view (&app->window,
	                                   &category->waiting_view);
	main_window_connect_download_view (&app->window,
	                                   &category->completed_view);
	main_window_connect_download_view (&app->window,
	                                   &category->recycled_view);
	g_signal_connect_swapped (category->waiting_view.self, "key-press-event",
	                          G_CALLBACK (on_key_press_event), app);
	g_signal_connect_swapped (category->completed_view.self, "key-press-event",
	                          G_CALLBACK (on_key_press_event), app);
	g_signal_connect_swapped (category->recycled_view.self, "key-press-event",
	                          G_CALLBACK (on_key_press_event), app);
}

