/*
	main_window.h



	2003  Plus Huang
 */

#ifndef __MAIN_WINDOW_H
#define __MAIN_WINDOW_H

#include <gtk/gtk.h>
#include "category_store.h"

#ifdef __cplusplus
extern "C" {
#endif

typedef struct _MainWindow   MainWindow;
typedef struct _FileMenu     FileMenu;
typedef struct _GlobalMenu   GlobalMenu;
typedef struct _CategoryMenu CategoryMenu;
typedef struct _DownloadMenu DownloadMenu;
typedef struct _HelpMenu     HelpMenu;

struct _FileMenu
{
	GtkMenu*    self;

	GtkWidget*    load;
	GtkWidget*    save;
	GtkWidget*    save_as;
	GtkWidget*    import_html;
	GtkWidget*    export;
};

struct _GlobalMenu
{
	GtkMenu*    self;

	GtkWidget*    start_job;
	GtkWidget*    stop_all;
	GtkWidget*    category_default;
	GtkWidget*    setting;
	GtkWidget*    about;
};

struct _CategoryMenu
{
	GtkMenu*    self;

	GtkWidget*  new_category;
	GtkWidget*  erase_category;
	GtkWidget*  properties;
};

struct _DownloadMenu
{
	GtkMenu*    self;

	GtkWidget*  new_download;
	GtkWidget*  add_batch;
	GtkWidget*  erase;
	GtkWidget*  move_to;
	GtkWidget*  move_top;
	GtkWidget*  move_bottom;
	GtkWidget*  start;
	GtkWidget*  stop;
	GtkWidget*  properties;
};

struct _HelpMenu
{
	GtkMenu* self;

	GtkWidget*  about_urlget;
};

void file_menu_init (FileMenu* fm);
void global_menu_init (GlobalMenu* gm);
void category_menu_init (CategoryMenu* cm);
void download_menu_init (DownloadMenu* dm);
void help_menu_init (HelpMenu* hm);

// MainWindow
struct _MainWindow {
	GtkWindow*    self;

	GtkWidget*    vbox_download;
	GtkWidget*    vbox;
	GtkPaned*     hpaned;
	GtkPaned*     vpaned;
	GtkTreeView*  category_view;
	GtkStatusbar* statusbar;
	// text view
	GtkTextView*  text_view;
	GtkTextTag*   text_tag;

	// menus
	GtkWidget*    menu_bar;
	FileMenu      file_menu;
	GlobalMenu    global_menu;
	CategoryMenu  category_menu;
	DownloadMenu  download_menu;
	HelpMenu      help_menu;

	// Download toolbar
	GtkWidget*   button_start;
	GtkWidget*   button_stop;
	GtkWidget*   button_new;
	GtkWidget*   button_batch;
	GtkWidget*   button_erase;
	GtkWidget*   button_property;
	GtkWidget*   button_move_top;
	GtkWidget*   button_move_bottom;

	// other data
	DownloadListView* current_download_view;
	Category*         current_category;
	CategoryStore*    category_store;
};


void main_window_init (MainWindow* mw,
                       CategoryStore* c_store);
void main_window_set_download_view (MainWindow* mw,
                                    DownloadListView* dlv);

void main_window_decide_download_sensitive (MainWindow* mw);
void main_window_set_download_sensitive (MainWindow* mw, gboolean sens);
void main_window_set_category_sensitive (MainWindow* mw, gboolean sens);

void main_window_selection_changed (MainWindow* mw);

void main_window_text_view_update (MainWindow* mw);
void main_window_text_view_show (MainWindow* mw, Download* dl);
void main_window_text_view_decide_update (MainWindow* mw, GtkTreePath* path);

void main_window_connect_download_view (MainWindow* mw,
                                        DownloadListView* dlv);

void main_window_on_category_erasing (MainWindow* mw, Category* erasing);

#ifdef __cplusplus
}
#endif

#endif  // End of __MAIN_WINDOW_H

