//  proxy_setting.h
//
//  Plus Huang

#include <string.h>
#include <stdlib.h>
#include "proxy_setting.h"
#include "urlget.h"

void setting_set_string (gchar** str_addr, const gchar* src)
{
	gchar* old_str = *str_addr;

	*str_addr = (src!=NULL && *src!=0)? g_strdup (src) : NULL;
	g_free(old_str);
}

void setting_set_string_len (gchar** str_addr, const gchar* src, int len)
{
	gchar* old_str = *str_addr;

	*str_addr = (src!=NULL && *src!=0)? g_strndup (src, len) : NULL;
	g_free(old_str);
}


/* ProxySetting ============================================================ */
ProxySetting* proxy_setting_init (ProxySetting* pa)
{
	pa->type = PROXY_TYPE_NONE;
	pa->port = 80;
	pa->address = NULL;

	return pa;
}

void proxy_setting_clear (ProxySetting* pa)
{
	g_free (pa->address);
	pa->address = NULL;
}

void proxy_setting_destroy (ProxySetting* pa)
{
	proxy_setting_clear (pa);
	g_free (pa);
}

void proxy_setting_assign (ProxySetting* pa, ProxySetting* src)
{
	pa->type    = src->type;
	pa->port    = src->port;
	proxy_setting_set_address(pa, src->address);
}

void proxy_setting_parse_xml_attr (ProxySetting* setting,
                                   const char** attr_names,
                                   const char** attr_values)
{
	int ii;

	for (ii=0; attr_names[ii]; ii++) {
		if (strcmp (attr_names[ii], "type")==0)
			setting->type = atoi (attr_values[ii]);
		if (strcmp (attr_names[ii], "port")==0)
			setting->port = atoi (attr_values[ii]);

		if (strcmp (attr_names[ii], "address")==0)
			setting_set_string (&setting->address, attr_values[ii]);
	}
}

void proxy_setting_write_xml (ProxySetting* setting,
                              FILE* file, int level)
{
	fprintf (file,
	         "%*c<" PROXY_SETTING_TAG " "         // Start of format string
	         "type=\"%d\" "
	         "port=\"%d\" "
	         "address=\"%s\" />\n",   // End of format string
	         level, ' ',
	         setting->type,
	         setting->port,
	         STR_N2E (setting->address));
}

