#include "proxy_setting_view.h"
#include "urlget.h"

static void on_toggled(ProxySettingView* view)
{
	if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON(view->radio_none))) {
		gtk_widget_set_sensitive (view->entry_proxy, FALSE);
		gtk_widget_set_sensitive (view->spin_port, FALSE);
		gtk_widget_set_sensitive (view->label_proxy, FALSE);
		gtk_widget_set_sensitive (view->label_port, FALSE);
	}
	else {
		gtk_widget_set_sensitive (view->entry_proxy, TRUE);
		gtk_widget_set_sensitive (view->spin_port, TRUE);
		gtk_widget_set_sensitive (view->label_proxy, TRUE);
		gtk_widget_set_sensitive (view->label_port, TRUE);
	}
}

void proxy_setting_view_init (ProxySettingView* view)
{
	GtkWidget* hbox;
	GtkWidget* vbox;
	GtkSizeGroup* size_group;

	// class initial base class : SettingView
	setting_view_init (SETTING_VIEW (view));

	view->self = gtk_hbox_new (FALSE, 5);
	view->entry_proxy = gtk_entry_new ();
	view->spin_port = gtk_spin_button_new_with_range (0.0, 65535.0, 1.0);
	gtk_entry_set_width_chars (GTK_ENTRY(view->spin_port), 5);
	view->radio_none = gtk_radio_button_new_with_mnemonic (NULL,
	                                                       _("_Direct connection "));
	view->radio_http = gtk_radio_button_new_with_mnemonic_from_widget ((GtkRadioButton*)view->radio_none,
	                                                             "_HTTP");
	view->radio_socks4 = gtk_radio_button_new_with_mnemonic_from_widget ((GtkRadioButton*)view->radio_none,
	                                                               "SOCKS _v4");
	view->radio_socks5 = gtk_radio_button_new_with_mnemonic_from_widget ((GtkRadioButton*)view->radio_none,
	                                                               "_SOCKS v5");
	size_group = gtk_size_group_new (GTK_SIZE_GROUP_HORIZONTAL);

	vbox = gtk_vbox_new (FALSE, 5);
	gtk_box_pack_start (GTK_BOX(vbox),
	                    view->radio_none,
	                    FALSE, FALSE, 0);
	gtk_box_pack_start (GTK_BOX(vbox),
	                    view->radio_http,
	                    FALSE, FALSE, 0);
	gtk_box_pack_start (GTK_BOX(vbox),
	                    view->radio_socks4,
	                    FALSE, FALSE, 0);
	gtk_box_pack_start (GTK_BOX(vbox),
	                    view->radio_socks5,
	                    FALSE, FALSE, 0);
	gtk_box_pack_start (GTK_BOX(view->self),
	                    vbox,
	                    FALSE, FALSE, 0);

	gtk_box_pack_start (GTK_BOX(view->self), gtk_vseparator_new(),
	                    FALSE, FALSE, 0);

	vbox = gtk_vbox_new (FALSE, 5);
	hbox = gtk_hbox_new (FALSE, 5);
	view->label_proxy = gtk_label_new_with_mnemonic (_("_Proxy:"));
	gtk_label_set_mnemonic_widget (GTK_LABEL(view->label_proxy),
	                               view->entry_proxy);
	gtk_size_group_add_widget (size_group, view->label_proxy);
	gtk_box_pack_start (GTK_BOX(hbox),
	                    view->label_proxy,
	                    FALSE, FALSE, 0);
	gtk_box_pack_start (GTK_BOX(hbox),
	                    view->entry_proxy,
	                    TRUE, TRUE, 0);
	gtk_box_pack_start (GTK_BOX(vbox),
	                    hbox,
	                    FALSE, FALSE, 0);

	hbox = gtk_hbox_new (FALSE, 5);
	view->label_port = gtk_label_new_with_mnemonic (_(" P_ort:"));
	gtk_label_set_mnemonic_widget (GTK_LABEL(view->label_port),
	                               view->spin_port);
	gtk_size_group_add_widget (size_group, view->label_port);
	gtk_box_pack_start (GTK_BOX(hbox),
	                    view->label_port,
	                    FALSE, FALSE, 0);
	gtk_box_pack_start (GTK_BOX(hbox),
	                    view->spin_port,
	                    FALSE, FALSE, 0);
	gtk_box_pack_start (GTK_BOX(vbox),
	                    hbox,
	                    FALSE, FALSE, 0);
	gtk_box_pack_start (GTK_BOX(view->self),
	                    vbox,
	                    FALSE, FALSE, 0);

	// change setting view container
	SETTING_VIEW (view)->view_container = view->self;

	on_toggled (view);

	gtk_widget_show_all (view->self);

	g_signal_connect_swapped (view->radio_none, "toggled",
	                          G_CALLBACK(on_toggled), view);
}

void proxy_setting_view_set (ProxySettingView* view,
                             ProxySetting* setting)
{
	if (setting==NULL)
		return;

	gtk_entry_set_text (GTK_ENTRY(view->entry_proxy),
	                    STR_N2E(setting->address));
	gtk_spin_button_set_value (GTK_SPIN_BUTTON(view->spin_port),
	                           setting->port);

	switch( setting->type ) {
	case PROXY_TYPE_NONE:
		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(view->radio_none),
		                              TRUE);
		break;
	case PROXY_TYPE_HTTP:
		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(view->radio_http),
		                              TRUE);
		break;
	case PROXY_TYPE_SOCKS4:
		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(view->radio_socks4),
		                              TRUE);
		break;
	case PROXY_TYPE_SOCKS5:
		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(view->radio_socks5),
		                              TRUE);
		break;
	}
}

void proxy_setting_view_get (ProxySettingView* view,
                             ProxySetting* setting)
{
	proxy_setting_set_address (setting,
	                           gtk_entry_get_text (GTK_ENTRY(view->entry_proxy)));
	setting->port = gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON(view->spin_port));

	if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON(view->radio_none)))
		setting->type = PROXY_TYPE_NONE;
	else if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON(view->radio_http)))
		setting->type = PROXY_TYPE_HTTP;
	else if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON(view->radio_socks4)))
		setting->type = PROXY_TYPE_SOCKS4;
	else if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON(view->radio_socks5)))
		setting->type = PROXY_TYPE_SOCKS5;
}

void proxy_setting_view_set_multiple (ProxySettingView *view,
                                      gboolean multiple)
{
}

