#ifdef _WIN32
#include <windows.h>
#endif

#include <glib.h>
#include "urlget.h"

static const gchar* urlget_prog_dir = NULL;
static const gchar* urlget_conf_dir = NULL;
static const gchar* urlget_image_dir = NULL;
static const gchar* urlget_locale_dir = NULL;
static const gchar* urlget_data_filename = NULL;
static const gchar* urlget_conf_filename = NULL;

void urlget_init (int argc, char *argv[])
{
	const gchar* home_dir = g_get_home_dir ();
#ifdef _WIN32
	const gchar* prog_path = NULL;
	HMODULE hmod;

	hmod = GetModuleHandle(NULL);
	if( hmod ) {
		prog_path = g_malloc (256);
		GetModuleFileName( hmod, (LPTSTR)prog_path, 256 );
		urlget_prog_dir = g_path_get_dirname (prog_path);
		g_free ((gpointer)prog_path);
	}
	else
#endif
	{
		urlget_prog_dir = g_path_get_dirname (argv[0]);
	}

	urlget_conf_dir      = g_strdup_printf ("%s" G_DIR_SEPARATOR_S ".urlget",
	                                        home_dir);
	urlget_conf_filename = g_strdup_printf ("%s" G_DIR_SEPARATOR_S "setting.xml",
	                                        urlget_conf_dir);
	urlget_data_filename = g_strdup_printf ("%s" G_DIR_SEPARATOR_S "category.xml",
	                                        urlget_conf_dir);

#ifdef _WIN32
	urlget_image_dir = g_strdup_printf ("%s" G_DIR_SEPARATOR_S "pixmaps",
	                                    urlget_prog_dir);
	urlget_locale_dir = g_strdup_printf ("%s" G_DIR_SEPARATOR_S "locale",
	                                     urlget_prog_dir);
#else
# ifdef PACKAGE_PIXMAPS_DIR
	urlget_image_dir = PACKAGE_PIXMAPS_DIR;
# endif
# ifdef URLGET_LOCALEDIR
	urlget_locale_dir = URLGET_LOCALEDIR;
# endif
#endif
}

const gchar* urlget_get_prog_dir ()
{
	return urlget_prog_dir;
}

const gchar* urlget_get_conf_dir ()
{
	return urlget_conf_dir;
}

const gchar* urlget_get_image_dir ()
{
	return urlget_image_dir;
}

const gchar* urlget_get_locale_dir ()
{
	return urlget_locale_dir;
}

const gchar* urlget_get_data_filename ()
{
	return urlget_data_filename;
}

const gchar* urlget_get_conf_filename ()
{
	return urlget_conf_filename;
}

#ifdef _WIN32
// log handler
static void dummy_print( const gchar* string )
{
	return;
}

static void dummy_log_handler (const gchar    *domain,
                               GLogLevelFlags  flags,
                               const gchar    *msg,
                               gpointer        user_data)
{
	return;
}

// disable log and console output in win32
void urlget_disable_console_output ()
{
	g_log_set_handler (NULL,
	                   G_LOG_LEVEL_MASK | G_LOG_FLAG_FATAL | G_LOG_FLAG_RECURSION,
	                   dummy_log_handler, NULL);	
	g_log_set_handler ("Gdk",
	                   G_LOG_LEVEL_MASK | G_LOG_FLAG_FATAL | G_LOG_FLAG_RECURSION,
	                   dummy_log_handler, NULL);	
	g_log_set_handler ("Gtk",
	                   G_LOG_LEVEL_MASK | G_LOG_FLAG_FATAL | G_LOG_FLAG_RECURSION,
	                   dummy_log_handler, NULL);	
	g_log_set_handler ("GLib",
	                   G_LOG_LEVEL_MASK | G_LOG_FLAG_FATAL | G_LOG_FLAG_RECURSION,
	                   dummy_log_handler, NULL);	
	g_log_set_handler ("GModule",
	                   G_LOG_LEVEL_MASK | G_LOG_FLAG_FATAL | G_LOG_FLAG_RECURSION,
	                   dummy_log_handler, NULL);	
	g_log_set_handler ("GLib-GObject",
	                   G_LOG_LEVEL_MASK | G_LOG_FLAG_FATAL | G_LOG_FLAG_RECURSION,
	                   dummy_log_handler, NULL);	
	g_log_set_handler ("GThread",
	                   G_LOG_LEVEL_MASK | G_LOG_FLAG_FATAL | G_LOG_FLAG_RECURSION,
	                   dummy_log_handler, NULL);	

	g_set_print_handler( dummy_print );
}
#endif


