#include <string.h>
#include "utility.h"

// create_dir (gchar* path)
// return FALSE if error
#ifdef _WIN32
 #include <direct.h>
 #define create_dir(p)   (_mkdir (p) == 0)
#else
 #include <unistd.h>
 #include <sys/stat.h>
 #define create_dir(p)   ( mkdir (p, 0750) == 0 )
#endif

gboolean make_path_exist (const gchar* path)
{
	char *path_beg, *path_cur, *path_end;
	char save_char;
	int  return_val=TRUE;

	path_cur = path_beg = g_strdup(path);
	path_end = path_beg + strlen(path_beg);
	while( *path_cur ) {
		path_cur = strchr(path_cur, G_DIR_SEPARATOR);
		path_cur = (path_cur) ? path_cur : path_end-1;
		save_char = *(++path_cur);
		*path_cur = 0;
		if( g_file_test(path_beg, G_FILE_TEST_EXISTS)==FALSE
		    && create_dir (path_beg) == FALSE )
		{
			return_val=FALSE;
			break;
		}
		*path_cur = save_char;
	}
	g_free (path_beg);
	return return_val;
}


// Begin of UrlPart ------------------------------------------------
void url_part_read_port (UrlPart* up)
{
	char* s;

	if (*up->current == 0)
		return;

	for (s=up->current; 1; ++s) {
		if (s[0]==0 || s[0]=='/') {
			if (s!=up->current) {
				up->port_beg = up->current;
				up->port_end = s;
			}
			break;
		}
	}
	up->current = (s[0]) ? s+1 : s;
}

void url_part_read_address (UrlPart* up)
{
	char* s;

	if (*up->current == 0)
		return;

	for (s=up->current; 1; ++s) {
		if ( *s=='/' || *s==':' || *s==0 ) {
			if (s != up->current) {
				up->address_beg = up->current;
				up->address_end = s;
			}
			up->current = (*s==0) ? s: s+1;
			if (*s==':')
				url_part_read_port (up);
			break;
		}
	}
}

void url_part_read_protocol (UrlPart* up)
{
	char* s;

	for (s=up->current; *s ; ++s) {
		if ( *s==':' ) {
			if (s!=up->current) {
				up->protocol_beg = up->current;
				up->protocol_end = s;
			}
			up->current = (s[1] == '/' && s[2]=='/')? s+3 : s+1;
			break;
		}
		if ( (*s>='a' && *s<='z') || (*s>='A' && *s<='Z') )
			continue;
		return;
	}
	url_part_read_address (up);
}

void url_part (UrlPart* up, const char* str)
{
	char* s;

	up->protocol_end = NULL;
	up->protocol_beg = NULL;
	up->address_end  = NULL;
	up->address_beg  = NULL;
	up->port_end     = NULL;
	up->port_beg     = NULL;
	up->path_end     = NULL;
	up->path_beg     = NULL;
	up->folder_end   = NULL;
	up->folder_beg   = NULL;
	up->name_end     = NULL;
	up->name_beg     = NULL;
	up->ext_end      = NULL;
	up->ext_beg      = NULL;
	up->current      = (char*)str;

	if (str==NULL)
		return;

	url_part_read_protocol (up);

	if (*up->current==0)
		return;

	// full path name
	for (s=up->current; 1 ; ++s) {
		if( *s == 0 ) {
			up->path_beg=up->current;
			up->path_end=s;
			break;
		}
	}
	// folder
	for (s=up->path_end-1; s!=up->path_beg-1 ; --s) {
		if ( *s == '/' ) {
			up->folder_beg = up->current;
			up->folder_end = s;
			up->current = up->folder_end+1;
			break;
		}
	}

	// name & ext
	if (*up->current==0)
		return;
	up->name_beg = up->current;
	up->name_end = up->path_end;
	for (s=up->current; *s!=0; ++s) {
		if ( *s == '.' ) {
			up->name_end = s;
			up->ext_beg = s+1;
			up->ext_end = up->path_end;
		}
	}
}

// End of UrlPart ------------------------------------------------

