#ifndef __XML_CONF_PARSER_H
#define __XML_CONF_PARSER_H

#include <glib.h>
#include "global_setting.h"

#ifdef __cplusplus
extern "C" {
#endif

typedef struct _XmlConfParser XmlConfParser;

struct _XmlConfParser {
	GMarkupParser        parser;
	GMarkupParseContext* parser_context;

	const gchar*         element_name;
	const gchar**        attr_names;
	const gchar**        attr_values;
	const gchar*         text;
	guint                text_len;
	gboolean             closing_tag;
	gint                 user_define;

	GPtrArray*           parser_array;
};

typedef void (*xml_conf_parser_func) (XmlConfParser* xcp,
                                      gpointer       pointer);

XmlConfParser* xml_conf_parser_new ();
void xml_conf_parser_destroy (XmlConfParser* xcp);

void xml_conf_parser_init (XmlConfParser* xcp);
void xml_conf_parser_clear (XmlConfParser* xcp);


gboolean xml_conf_parser_read_file (XmlConfParser* xcp,
                                    const gchar* filename,
                                    GError** error);

// parser stack
void xml_conf_parser_stack_call (XmlConfParser*  xcp);

void xml_conf_parser_stack_push (XmlConfParser*       xcp,
                                 xml_conf_parser_func func,
                                 gpointer             data);

void xml_conf_parser_stack_pop (XmlConfParser*  xcp);


// setting parser
GlobalSetting* xml_conf_parser_read_setting (XmlConfParser* xcp,
                                             const gchar* filename,
                                             GError** error);
void xml_conf_parse_proxy_setting (XmlConfParser* xcp, gpointer pointer);
void xml_conf_parse_download_setting (XmlConfParser* xcp, gpointer pointer);
void xml_conf_parse_category_setting (XmlConfParser* xcp, gpointer pointer);
void xml_conf_parse_global_setting (XmlConfParser* xcp, gpointer pointer);

// category parser
GList* xml_conf_parser_read_category (XmlConfParser* xcp,
                                      const gchar* filename,
                                      GError** error);

#ifdef __cplusplus
}
#endif

#endif  // End of __XML_CONF_PARSER_H
