#include <string.h>
#include "xml_conf_parser.h"
#include "category_store.h"

void xml_conf_parse_download_store (XmlConfParser* xcp,
                                    DownloadStore* store)
{
	Download* dl_data;

	if (xcp->element_name == NULL)
		return;

	if (strcmp (xcp->element_name, DOWNLOAD_SETTING_TAG)==0)
	{
		dl_data = download_new ();
		download_store_add_static (store, dl_data);

		xml_conf_parser_stack_push (xcp, xml_conf_parse_download_setting, dl_data);
		xml_conf_parse_download_setting (xcp, dl_data);
	}
}

void xml_conf_parse_category (XmlConfParser* xcp,
                              Category* category)
{
	DownloadStore*   download_store;

	if (xcp->element_name == NULL)
		return;

	if (strcmp (xcp->element_name, CATEGORY_SETTING_TAG)==0) {
		xml_conf_parser_stack_push (xcp,
		                            xml_conf_parse_category_setting,
		                            category);
		xml_conf_parse_category_setting (xcp, category);
	}
	else {
		if (strcmp (xcp->element_name, CATEGORY_WAITING_STORE_TAG)==0 )
			download_store = &category->waiting_store;
		else if (strcmp (xcp->element_name, CATEGORY_COMPLETED_STORE_TAG)==0 )
			download_store = &category->completed_store;
		else if (strcmp (xcp->element_name, CATEGORY_RECYCLED_STORE_TAG)==0 )
			download_store = &category->recycled_store;

		xml_conf_parser_stack_push (xcp,
		                            (xml_conf_parser_func)xml_conf_parse_download_store,
		                            download_store);
	}
}

void xml_conf_parse_category_store (XmlConfParser* xcp,
                                    GList** list)
{
	Category*        category;

	if (xcp->element_name == NULL)
		return;

	if (strcmp (xcp->element_name, CATEGORY_STORE_TAG)==0) {
		xml_conf_parser_stack_push (xcp,
		                            (xml_conf_parser_func)xml_conf_parse_category_store,
		                            (gpointer)list);
	}
	else if (strcmp (xcp->element_name, CATEGORY_TAG)==0) {
		category = category_new ();
		*list = g_list_append (*list, category);

		xml_conf_parser_stack_push (xcp,
		                            (xml_conf_parser_func)xml_conf_parse_category,
		                            (gpointer)category);
	}
}

GList* xml_conf_parser_read_category (XmlConfParser* xcp,
                                      const gchar* filename,
                                      GError** error)
{
	GList* list = NULL;
	gboolean read_ok;

	xml_conf_parser_stack_push (xcp,
	                            (xml_conf_parser_func)xml_conf_parse_category_store,
	                            &list);

	read_ok = xml_conf_parser_read_file (xcp, filename, error);

	return list;
}
