#include <string.h>
#include "xml_conf_parser.h"

void xml_conf_parse_download_text (XmlConfParser* xcp, DownloadSetting* ds);
void xml_conf_parse_global_text (XmlConfParser* xcp, gpointer pointer);

GlobalSetting* xml_conf_parser_read_setting (XmlConfParser* xcp,
                                             const gchar* filename,
                                             GError** error)
{
	GlobalSetting* data = global_setting_new ();

	xml_conf_parser_stack_push (xcp, xml_conf_parse_global_setting, data);

	if (xml_conf_parser_read_file (xcp, filename, error))
		return data;

	global_setting_destroy (data);

	return NULL;
}

void xml_conf_parse_proxy_setting (XmlConfParser* xcp, gpointer pointer)
{
	if (xcp->element_name == NULL)
		return;

	proxy_setting_parse_xml_attr (pointer,
	                              xcp->attr_names,
	                              xcp->attr_values);
}

void xml_conf_parse_download_setting (XmlConfParser* xcp, gpointer pointer)
{
	if (xcp->element_name == NULL)
		return;

	if (strcmp (xcp->element_name, PROXY_SETTING_TAG)==0) {
		xml_conf_parser_stack_push (xcp, xml_conf_parse_proxy_setting, pointer);
		xml_conf_parse_proxy_setting (xcp, pointer);
	}
	else if (strcmp (xcp->element_name, "url")==0) {
		xml_conf_parser_stack_push (xcp, xml_conf_parse_download_text, pointer);
		xcp->user_define = 1;
	}
	else if (strcmp (xcp->element_name, "directory")==0) {
		xml_conf_parser_stack_push (xcp, xml_conf_parse_download_text, pointer);
		xcp->user_define = 2;
	}
	else if (strcmp (xcp->element_name, "filename")==0) {
		xml_conf_parser_stack_push (xcp, xml_conf_parse_download_text, pointer);
		xcp->user_define = 3;
	}
	else
		download_setting_parse_xml_attr (pointer,
		                                 xcp->attr_names,
		                                 xcp->attr_values);
}

void xml_conf_parse_download_text (XmlConfParser* xcp, DownloadSetting* ds)
{
	switch (xcp->user_define) {
	case 1:
		setting_set_string_len (&ds->url, xcp->text, xcp->text_len);
		break;
	case 2:
		setting_set_string_len (&ds->directory, xcp->text, xcp->text_len);
		break;
	case 3:
		setting_set_string_len (&ds->filename, xcp->text, xcp->text_len);
		break;
	}
}

void xml_conf_parse_category_setting (XmlConfParser* xcp, gpointer pointer)
{
	if (xcp->element_name == NULL)
		return;

	if (strcmp (xcp->element_name, DOWNLOAD_SETTING_TAG)==0) {
		xml_conf_parser_stack_push (xcp, xml_conf_parse_download_setting, pointer);
		xml_conf_parse_download_setting (xcp, pointer);
	}
	else
		category_setting_parse_xml_attr (pointer,
		                                 xcp->attr_names,
		                                 xcp->attr_values);
}

void xml_conf_parse_global_setting (XmlConfParser* xcp, gpointer pointer)
{
	if (xcp->element_name == NULL)
		return;

	if (strcmp (xcp->element_name, CATEGORY_SETTING_TAG)==0) {
		xml_conf_parser_stack_push (xcp, xml_conf_parse_category_setting, pointer);
		xml_conf_parse_category_setting (xcp, pointer);
	}
	else if (strcmp (xcp->element_name, "directory")==0) {
		xml_conf_parser_stack_push (xcp, xml_conf_parse_global_text, pointer);
	}
	else {
		global_setting_parse_xml_attr (pointer,
		                               xcp->attr_names,
		                               xcp->attr_values);
	}
}

void xml_conf_parse_global_text (XmlConfParser* xcp, gpointer pointer)
{
	gchar* str = g_strndup (xcp->text, xcp->text_len);

	global_setting_add_directory (pointer, str);
	g_free (str);

//	xml_conf_parser_stack_pop (xcp);
}

